@@ -31,8 +31,10 @@
 #include "../e-timezone-entry.h"
 #include "../calendar-config.h"
 #include "cal-prefs-dialog.h"
+#include <libgnomeui/gnome-color-picker.h>
 #include <widgets/misc/e-dateedit.h>
 #include <e-util/e-dialog-widgets.h>
+#include <e-util/e-util-private.h>
 #include <libgnome/gnome-i18n.h>
 #include <string.h>
 
@@ -100,10 +102,10 @@ static const char *
 spec_from_picker (GtkWidget *picker)
 {
 	static char spec[8];
-	GdkColor color;
+	guint8 r, g, b;
 
-	gtk_color_button_get_color (GTK_COLOR_BUTTON (picker), &color);
-	g_snprintf (spec, sizeof (spec), "#%02x%02x%02x", color.red, color.green, color.blue);
+	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (picker), &r, &g, &b, NULL);
+	g_snprintf (spec, sizeof (spec), "#%02x%02x%02x", r, g, b);
 
 	return spec;
 }
@@ -187,7 +189,7 @@ week_start_day_changed (GtkWidget *widget, CalendarPrefsDialog *prefs)
 {
 	int week_start_day;
 
-	week_start_day = e_dialog_option_menu_get (prefs->week_start_day, week_start_day_map);
+	week_start_day = e_dialog_combo_box_get (prefs->week_start_day, week_start_day_map);
 	calendar_config_set_week_start_day (week_start_day);
 }
 
@@ -209,7 +211,7 @@ time_divisions_changed (GtkWidget *widget, CalendarPrefsDialog *prefs)
 {
 	int time_divisions;
 
-	time_divisions = e_dialog_option_menu_get (prefs->time_divisions, time_division_map);
+	time_divisions = e_dialog_combo_box_get (prefs->time_divisions, time_division_map);
 	calendar_config_set_time_divisions (time_divisions);
 }
 
@@ -258,13 +260,13 @@ hide_completed_tasks_units_changed (GtkWidget *widget, CalendarPrefsDialog *pref
 }
 
 static void
-tasks_due_today_set_color (GtkColorButton *picker, guint r, guint g, guint b, guint a, CalendarPrefsDialog *prefs)
+tasks_due_today_set_color (GnomeColorPicker *picker, guint r, guint g, guint b, guint a, CalendarPrefsDialog *prefs)
 {
 	calendar_config_set_tasks_due_today_color (spec_from_picker (prefs->tasks_due_today_color));
 }
 
 static void
-tasks_overdue_set_color (GtkColorButton *picker, guint r, guint g, guint b, guint a, CalendarPrefsDialog *prefs)
+tasks_overdue_set_color (GnomeColorPicker *picker, guint r, guint g, guint b, guint a, CalendarPrefsDialog *prefs)
 {
 	calendar_config_set_tasks_overdue_color (spec_from_picker (prefs->tasks_overdue_color));
 }
@@ -284,15 +286,54 @@ default_reminder_toggled (GtkToggleButton *toggle, CalendarPrefsDialog *prefs)
 static void
 default_reminder_interval_changed (GtkWidget *widget, CalendarPrefsDialog *prefs)
 {
-	calendar_config_set_default_reminder_interval (
-		e_dialog_spin_get_int (prefs->default_reminder_interval));
+	const gchar *str;
+	gdouble value;
+
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	value = g_ascii_strtod (str, NULL);
+
+	calendar_config_set_default_reminder_interval (value);
 }
 
 static void
 default_reminder_units_changed (GtkWidget *widget, CalendarPrefsDialog *prefs)
 {
 	calendar_config_set_default_reminder_units (
-		e_dialog_option_menu_get (prefs->default_reminder_units, default_reminder_units_map));
+		e_dialog_combo_box_get (prefs->default_reminder_units, default_reminder_units_map));
+}
+
+static void
+alarms_selection_changed (ESourceSelector *selector, CalendarPrefsDialog *prefs)
+{
+	ESourceList *source_list = prefs->alarms_list;
+	GSList *selection;
+	GSList *l;
+	GSList *groups;
+
+	/* first we clear all the alarm flags from all sources */
+	g_message ("Clearing selection");
+	for (groups = e_source_list_peek_groups (source_list); groups; groups = groups->next) {
+		ESourceGroup *group = E_SOURCE_GROUP (groups->data);
+		GSList *sources;
+		for (sources = e_source_group_peek_sources (group); sources; sources = sources->next) {
+			ESource *source = E_SOURCE (sources->data);
+
+			g_message ("Unsetting for %s", e_source_peek_name (source));
+			e_source_set_property (source, "alarm", "false");
+		}
+	}
+
+	/* then we loop over the selector's selection, setting the
+	   property on those sources */
+	selection = e_source_selector_get_selection (selector);
+	for (l = selection; l; l = l->next) {
+		g_message ("Setting for %s", e_source_peek_name (E_SOURCE (l->data)));
+		e_source_set_property (E_SOURCE (l->data), "alarm", "true");
+	}
+	e_source_selector_free_selection (selection);
+
+	/* FIXME show an error if this fails? */
+	e_source_list_sync (source_list, NULL);	
 }
 
 static void
@@ -340,6 +381,9 @@ setup_changes (CalendarPrefsDialog *prefs)
 			  G_CALLBACK (default_reminder_interval_changed), prefs);
 	g_signal_connect (G_OBJECT (prefs->default_reminder_units), "changed", G_CALLBACK (default_reminder_units_changed), prefs);
 
+	g_signal_connect (G_OBJECT (prefs->alarm_list_widget), "selection_changed", G_CALLBACK (alarms_selection_changed), prefs);
+	
+
 	g_signal_connect (G_OBJECT (prefs->template_url), "changed", G_CALLBACK (template_url_changed), prefs);
 }
 
@@ -352,7 +396,11 @@ set_color_picker (GtkWidget *picker, const char *spec)
 	if (!spec || !gdk_color_parse (spec, &color))
 		color.red = color.green = color.blue = 0;
 
-	gtk_color_button_set_color (GTK_COLOR_BUTTON (picker), &color);
+	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (picker),
+				    color.red,
+				    color.green,
+				    color.blue,
+				    65535);
 }
 
 /* Shows the current Free/Busy settings in the dialog */
@@ -391,6 +439,38 @@ show_task_list_config (CalendarPrefsDialog *prefs)
 	gtk_widget_set_sensitive (prefs->tasks_hide_completed_units, hide_completed_tasks);
 }
 
+static void
+initialize_selection (ESourceSelector *selector, ESourceList *source_list)
+{
+	GSList *groups;
+
+	for (groups = e_source_list_peek_groups (source_list); groups; groups = groups->next) {
+		ESourceGroup *group = E_SOURCE_GROUP (groups->data);
+		GSList *sources;
+		for (sources = e_source_group_peek_sources (group); sources; sources = sources->next) {
+			ESource *source = E_SOURCE (sources->data);
+			const char *completion = e_source_get_property (source, "alarm");
+			if (!completion  || !g_ascii_strcasecmp (completion, "true")) {
+				if (!completion)
+					e_source_set_property (E_SOURCE (source), "alarm", "true");
+				e_source_selector_select_source (selector, source);
+			}
+		}
+	}
+}
+
+static void
+show_alarms_config (CalendarPrefsDialog *prefs)
+{
+	if (!e_cal_get_sources (&prefs->alarms_list, E_CAL_SOURCE_TYPE_EVENT, NULL))
+		return;
+
+	prefs->alarm_list_widget = e_source_selector_new (prefs->alarms_list);
+	atk_object_set_name (gtk_widget_get_accessible (prefs->alarm_list_widget), _("Selected Calendars for Alarms"));
+	gtk_container_add (GTK_CONTAINER (prefs->scrolled_window), prefs->alarm_list_widget);
+	initialize_selection (prefs->alarm_list_widget, prefs->alarms_list);
+}
+
 /* Shows the current config settings in the dialog. */
 static void
 show_config (CalendarPrefsDialog *prefs)
@@ -449,6 +529,9 @@ show_config (CalendarPrefsDialog *prefs)
 	/* Task list */
 	show_task_list_config (prefs);
 
+	/* Alarms list*/
+	show_alarms_config (prefs);
+
 	/* Free/Busy */
 	show_fb_config (prefs);
 
@@ -469,6 +552,7 @@ static ECalConfigItem eccp_items[] = {
 	{ E_CONFIG_PAGE,          "10.display",                   "display",             eccp_widget_glade },
 	{ E_CONFIG_SECTION,       "10.display/00.general",        "displayGeneral",      eccp_widget_glade },
 	{ E_CONFIG_SECTION,       "10.display/10.taskList",       "taskList",            eccp_widget_glade },
+	{ E_CONFIG_PAGE,	  "15.alarms",			  "alarms",		 eccp_widget_glade },
 	{ E_CONFIG_PAGE,          "20.freeBusy",                  "freebusy",            eccp_widget_glade },
 	{ E_CONFIG_SECTION,       "20.freeBusy/00.defaultServer", "defaultFBServer",     eccp_widget_glade },
 };
@@ -497,8 +581,13 @@ calendar_prefs_dialog_construct (CalendarPrefsDialog *prefs)
 		"fri_button",
 		"sat_button",
 	};
+	char *gladefile;
 
-	gui = glade_xml_new (EVOLUTION_GLADEDIR "/cal-prefs-dialog.glade", "toplevel-notebook", NULL);
+	gladefile = g_build_filename (EVOLUTION_GLADEDIR,
+				      "cal-prefs-dialog.glade",
+				      NULL);
+	gui = glade_xml_new (gladefile, "toplevel-notebook", NULL);
+	g_free (gladefile);
 	prefs->gui = gui;
 
 	prefs->gconf = gconf_client_get_default ();
@@ -542,6 +631,10 @@ calendar_prefs_dialog_construct (CalendarPrefsDialog *prefs)
 	prefs->tasks_hide_completed_interval = glade_xml_get_widget (gui, "tasks_hide_completed_interval");
 	prefs->tasks_hide_completed_units = glade_xml_get_widget (gui, "tasks_hide_completed_units");
 
+
+	/* Alarms tab */
+	prefs->scrolled_window = glade_xml_get_widget (gui, "calendar-source-scrolled-window");
+	
 	/* Free/Busy tab */
 	prefs->template_url = glade_xml_get_widget (gui, "template_url");
 	target = e_cal_config_target_new_prefs (ec, prefs->gconf);
@@ -595,6 +688,7 @@ cal_prefs_dialog_create_time_edit (void)
 
 	dedit = e_date_edit_new ();
 
+	gtk_widget_show (GTK_WIDGET (dedit));
 	e_date_edit_set_use_24_hour_format (E_DATE_EDIT (dedit), calendar_config_get_24_hour_format ());
 	e_date_edit_set_time_popup_range (E_DATE_EDIT (dedit), 0, 24);
 	e_date_edit_set_show_date (E_DATE_EDIT (dedit), FALSE);
