@@ -43,6 +43,14 @@ typedef struct {
 	struct icaltimetype *complete;
 } CompEditorPageDates;
 
+typedef enum {
+	COMP_EDITOR_PAGE_NEW_ITEM = 1<<0,
+	COMP_EDITOR_PAGE_MEETING = 1<<1,
+	COMP_EDITOR_PAGE_DELEGATE = 1<<2,
+	COMP_EDITOR_PAGE_USER_ORG = 1<<3,
+	COMP_EDITOR_PAGE_IS_ASSIGNED = 1<<4,
+} CompEditorPageFlags;
+
 typedef struct {
 	GtkObject object;
 
@@ -55,6 +63,9 @@ typedef struct {
 	   normally, but we create our pages individually so have to do it
 	   ourselves. */
 	GtkAccelGroup *accel_group;
+
+	CompEditorPageFlags flags;
+
 } CompEditorPage;
 
 typedef struct {
@@ -68,7 +79,8 @@ typedef struct {
 	void (* summary_changed) (CompEditorPage *page, const char *summary);
 	void (* dates_changed)   (CompEditorPage *page, const char *dates);
 	void (* client_changed)  (CompEditorPage *page, ECal *client);
-
+	void (* focus_in) 	 (CompEditorPage *page, GtkWidget *widget);
+	void (* focus_out)	 (CompEditorPage *page, GtkWidget *widget);
 	/* Virtual methods */
 
 	GtkWidget *(* get_widget) (CompEditorPage *page);
@@ -82,7 +94,6 @@ typedef struct {
 	void (* set_dates) (CompEditorPage *page, CompEditorPageDates *dates);
 } CompEditorPageClass;
 
-
 GtkType    comp_editor_page_get_type               (void);
 GtkWidget *comp_editor_page_get_widget             (CompEditorPage      *page);
 void       comp_editor_page_focus_main_widget      (CompEditorPage      *page);
