@@ -406,6 +406,8 @@ sensitize_widgets (TaskPage *tpage)
 
 	gtk_widget_set_sensitive (priv->organizer, !read_only);
 	gtk_widget_set_sensitive (priv->add, (!read_only &&  sens));
+	gtk_widget_set_sensitive (priv->edit, (!read_only && sens));
+	e_meeting_list_view_set_editable (priv->list_view, (!read_only && sens));
 	gtk_widget_set_sensitive (priv->remove, (!read_only &&  sens));
 	gtk_widget_set_sensitive (priv->invite, (!read_only &&  sens));
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->list_view), !read_only);	
@@ -434,14 +436,14 @@ sensitize_widgets (TaskPage *tpage)
 		gtk_widget_hide (priv->attendee_box);
 		gtk_widget_hide (priv->organizer);
 		gtk_widget_hide (priv->invite);
-		gtk_label_set_text_with_mnemonic ((GtkLabel *) priv->org_cal_label, _("_Group"));
+		gtk_label_set_text_with_mnemonic ((GtkLabel *) priv->org_cal_label, _("_Group:"));
 	} else {
 		gtk_widget_show (priv->invite);	
 		gtk_widget_show (priv->calendar_label);
 		gtk_widget_show (priv->list_box);
 		gtk_widget_show (priv->attendee_box);
 		gtk_widget_show (priv->organizer);
-		gtk_label_set_text_with_mnemonic ((GtkLabel *) priv->org_cal_label, _("Or_ganizer"));		
+		gtk_label_set_text_with_mnemonic ((GtkLabel *) priv->org_cal_label, _("Or_ganizer:"));		
 	}
 }
 void
@@ -595,7 +597,7 @@ task_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 	
 	e_timezone_entry_set_timezone (E_TIMEZONE_ENTRY (priv->timezone),
 				       zone ? zone : default_zone);
-	gtk_widget_show_all (priv->timezone);
+	task_page_set_show_timezone (tpage, calendar_config_get_show_timezone());
 
 	if (!(COMP_EDITOR_PAGE (tpage)->flags & COMP_EDITOR_PAGE_NEW_ITEM) && !zone) {
 			task_page_set_show_timezone (tpage, FALSE);
@@ -684,6 +686,7 @@ task_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 								CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS))
 						gtk_widget_set_sensitive (priv->invite, FALSE);
 					gtk_widget_set_sensitive (priv->add, FALSE);
+					gtk_widget_set_sensitive (priv->edit, FALSE);
 					gtk_widget_set_sensitive (priv->remove, FALSE);
 					priv->user_org = FALSE;
 				}
@@ -978,6 +981,25 @@ add_clicked_cb (GtkButton *btn, TaskPage *page)
 	e_meeting_list_view_edit (page->priv->list_view, attendee);
 }
 
+edit_clicked_cb (GtkButton *btn, TaskPage *tpage)
+{
+	TaskPagePrivate *priv;
+	EMeetingAttendee *attendee;
+	GtkTreePath *path = NULL;
+	GtkTreeViewColumn *focus_col;
+	gint row = 0;
+
+	priv = tpage->priv;
+
+	gtk_tree_view_get_cursor (GTK_TREE_VIEW (priv->list_view), &path, NULL);
+	g_return_if_fail (path != NULL);
+
+	gtk_tree_view_get_cursor (GTK_TREE_VIEW (priv->list_view), &path, &focus_col);
+	gtk_tree_view_set_cursor (GTK_TREE_VIEW (priv->list_view), path, focus_col, TRUE);
+	gtk_tree_path_free (path);
+}
+
+
 static gboolean
 existing_attendee (EMeetingAttendee *ia, ECalComponent *comp) 
 {
@@ -1146,6 +1168,7 @@ attendee_added_cb (EMeetingListView *emlv, EMeetingAttendee *ia, gpointer user_d
 
 			   gtk_widget_set_sensitive (priv->invite, FALSE);
 			   gtk_widget_set_sensitive (priv->add, FALSE);
+			   gtk_widget_set_sensitive (priv->edit, FALSE);
 		   }
 	   }
    }
@@ -1181,7 +1204,7 @@ enum {
 
 static EPopupItem context_menu_items[] = {
 	{ E_POPUP_ITEM, "10.delete", N_("_Remove"), popup_delete_cb, NULL, GTK_STOCK_REMOVE, ATTENDEE_CAN_DELETE },
-	{ E_POPUP_ITEM, "15.add", N_("_Add "), popup_add_cb, NULL, GTK_STOCK_ADD },	
+	{ E_POPUP_ITEM, "15.add", N_("_Add "), popup_add_cb, NULL, GTK_STOCK_ADD, ATTENDEE_CAN_ADD },	
 };
 
 static void
@@ -1410,6 +1433,7 @@ get_widgets (TaskPage *tpage)
 	priv->organizer = GW ("organizer");
 	priv->invite = GW ("invite");
 	priv->add = GW ("add-attendee");
+	priv->edit = GW ("edit-attendee");
 	priv->remove = GW ("remove-attendee");
 	priv->list_box = GW ("list-box");
 	priv->calendar_label = GW ("group-label");
@@ -1565,7 +1589,28 @@ categories_clicked_cb (GtkWidget *button, gpointer data)
 	entry = priv->categories;
 	e_categories_config_open_dialog_for_entry (GTK_ENTRY (entry));
 }
+/* sets the current focused widget */
+static gboolean 
+widget_focus_in_cb (GtkWidget *widget,  GdkEventFocus *event, gpointer data)
+{
+	TaskPage *tpage;
+	tpage = TASK_PAGE  (data);
 
+	comp_editor_page_set_focused_widget (COMP_EDITOR_PAGE(tpage), widget);
+
+	return FALSE;
+}
+/* unsets the current focused widget */
+static gboolean
+widget_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+	TaskPage *tpage;
+	tpage = TASK_PAGE (data);
+
+	comp_editor_page_unset_focused_widget (COMP_EDITOR_PAGE(tpage), widget);
+
+	return FALSE;
+}
 /* This is called when any field is changed; it notifies upstream. */
 static void
 field_changed_cb (GtkWidget *widget, gpointer data)
@@ -1703,6 +1748,18 @@ init_widgets (TaskPage *tpage)
 	/* Connect the default signal handler to use to make sure the "changed"
 	   field gets set whenever a field is changed. */
 
+	/* Set the current focus entry */
+	g_signal_connect (priv->summary, "focus-in-event",
+			  G_CALLBACK (widget_focus_in_cb), tpage);
+	g_signal_connect (priv->summary, "focus-out-event",
+			  G_CALLBACK (widget_focus_out_cb), tpage);
+
+	g_signal_connect (priv->description, "focus-in-event",
+			  G_CALLBACK (widget_focus_in_cb), tpage);
+	g_signal_connect (priv->description, "focus-out-event",
+			  G_CALLBACK (widget_focus_out_cb), tpage);
+
+
 	/* Belongs to priv->description */
 	g_signal_connect ((text_buffer), "changed",
 			  G_CALLBACK (field_changed_cb), tpage);
@@ -1724,6 +1781,9 @@ init_widgets (TaskPage *tpage)
 	/* Add attendee button */
 	g_signal_connect (priv->add, "clicked", G_CALLBACK (add_clicked_cb), tpage);
 
+	/* Edit attendee button */
+	g_signal_connect (priv->edit, "clicked", G_CALLBACK (edit_clicked_cb), tpage);
+
 	/* Remove attendee button */
 	g_signal_connect (priv->remove, "clicked", G_CALLBACK (remove_clicked_cb), tpage);
 
@@ -1746,21 +1806,7 @@ init_widgets (TaskPage *tpage)
 	e_meeting_list_view_column_set_visible (priv->list_view, "Status", calendar_config_get_show_status());
 	e_meeting_list_view_column_set_visible (priv->list_view, "Type", calendar_config_get_show_type());
 	
-	if (!calendar_config_get_show_timezone()) {
-		gtk_widget_hide (priv->timezone_label);
-		gtk_widget_hide (priv->timezone);
-	} else {
-		gtk_widget_show (priv->timezone_label);
-		gtk_widget_show_all (priv->timezone);
-	}
-
-	if (!calendar_config_get_show_categories()) {
-		gtk_widget_hide (priv->categories_btn);
-		gtk_widget_hide (priv->categories);
-	} else {
-		gtk_widget_show (priv->categories_btn);
-		gtk_widget_show (priv->categories);
-	}	
+	task_page_set_show_categories (tpage, calendar_config_get_show_categories());
 
 	return TRUE;
 }
