@@ -165,8 +165,10 @@ write_html (GtkHTMLStream *stream, ECal *ecal, ECalComponent *comp, icaltimezone
 
 			icon_file = e_categories_get_icon_file_for ((const char *) node->data);
 			if (icon_file) {
-				gtk_html_stream_printf (stream, "<IMG ALT=\"%s\" SRC=\"file://%s\">",
-							(const char *) node->data, icon_file);
+				gchar *icon_file_uri = g_filename_to_uri (icon_file, NULL, NULL);
+				gtk_html_stream_printf (stream, "<IMG ALT=\"%s\" SRC=\"%s\">",
+							(const char *) node->data, icon_file_uri);
+				g_free (icon_file_uri);
 			}
 		}
 
@@ -229,7 +231,7 @@ write_html (GtkHTMLStream *stream, ECal *ecal, ECalComponent *comp, icaltimezone
 		str = g_strdup (_("Completed"));
 		break;
 	case ICAL_STATUS_CANCELLED :
-		str = g_strdup (_("Cancelled"));
+		str = g_strdup (_("Canceled"));
 		break;
 	case ICAL_STATUS_NONE :
 	default :
