@@ -20,18 +20,18 @@
  * Author: Rodrigo Moya <rodrigo@ximian.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
@@ -41,15 +41,26 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkprogressbar.h>
 #include <libecal/e-cal.h>
-#include <e-util/e-bconf-map.h>
-#include <e-util/e-folder-map.h>
+
 #include <libedataserver/e-dbhash.h>
 #include <libedataserver/e-xml-hash-utils.h>
-#include "calendar-config.h"
+#include <libedataserver/e-xml-utils.h>
+
+#include "e-util/e-bconf-map.h"
+#include "e-util/e-folder-map.h"
+#include "e-util/e-util-private.h"
+
 #include "calendar-config-keys.h"
+#include "calendar-config.h"
 #include "e-cal-event.h"
 #include "migration.h"
 
+#ifndef G_OS_WIN32
+
+/* No previous versions have been available on Win32, so don't
+ * bother with upgrade support from 1.x on Win32.
+ */
+
 static e_gconf_map_t calendar_display_map[] = {
 	/* /Calendar/Display */
 	{ "Timezone", "calendar/display/timezone", E_GCONF_MAP_STRING },
@@ -325,7 +336,7 @@ migrate_ical_folder_to_source (char *old_path, ESource *new_source, ECalSourceTy
 		g_warning ("could not find a backend for '%s'", e_source_get_uri (old_source));
 		goto finish;
 	}
-	if (!e_cal_open (old_ecal, TRUE, &error)) {
+	if (!e_cal_open (old_ecal, FALSE, &error)) {
 		g_warning ("failed to load source ecal for migration: '%s' (%s)", error->message,
 			   e_source_get_uri (old_source));
 		goto finish;
@@ -372,6 +383,7 @@ migrate_ical_folder (char *old_path, ESourceGroup *dest_group, char *source_name
 	return retval;
 }
 
+#endif	/* !G_OS_WIN32 */
 
 #define WEBCAL_BASE_URI "webcal://"
 #define CONTACTS_BASE_URI "contacts://"
@@ -416,10 +428,10 @@ create_calendar_sources (CalendarComponent *component,
 	*personal_source = NULL;
 
 	base_uri = g_build_filename (calendar_component_peek_base_directory (component),
-				     "/calendar/local/",
+				     "calendar", "local",
 				     NULL);
 
-	base_uri_proto = g_strconcat ("file://", base_uri, NULL);
+	base_uri_proto = g_filename_to_uri (base_uri, NULL, NULL);
 
 	groups = e_source_list_peek_groups (source_list);
 	if (groups) {
@@ -520,7 +532,7 @@ create_task_sources (TasksComponent *component,
 	base_uri = g_build_filename (tasks_component_peek_base_directory (component),
 				     "tasks", "local", NULL);
 
-	base_uri_proto = g_strconcat ("file://", base_uri, NULL);
+	base_uri_proto = g_filename_to_uri (base_uri, NULL, NULL);
 
 	groups = e_source_list_peek_groups (source_list);
 	if (groups) {
@@ -589,6 +601,8 @@ create_task_sources (TasksComponent *component,
 	g_free (base_uri);
 }
 
+#ifndef G_OS_WIN32
+
 static void
 migrate_pilot_db_key (const char *key, gpointer user_data)
 {
@@ -601,12 +615,12 @@ static void
 migrate_pilot_data (const char *component, const char *conduit, const char *old_path, const char *new_path)
 {
 	char *changelog, *map;
-	struct dirent *dent;
+	const char *dent;
 	const char *ext;
 	char *filename;
-	DIR *dir;
+	GDir *dir;
 	
-	if (!(dir = opendir (old_path)))
+	if (!(dir = g_dir_open (old_path, 0, NULL)))
 		return;
 	
 	map = g_alloca (12 + strlen (conduit));
@@ -615,24 +629,24 @@ migrate_pilot_data (const char *component, const char *conduit, const char *old_
 	changelog = g_alloca (24 + strlen (conduit));
 	sprintf (changelog, "pilot-sync-evolution-%s-", conduit);
 	
-	while ((dent = readdir (dir))) {
-		if (!strncmp (dent->d_name, map, strlen (map)) &&
-		    ((ext = strrchr (dent->d_name, '.')) && !strcmp (ext, ".xml"))) {
+	while ((dent = g_dir_read_name (dir))) {
+		if (!strncmp (dent, map, strlen (map)) &&
+		    ((ext = strrchr (dent, '.')) && !strcmp (ext, ".xml"))) {
 			/* pilot map file - src and dest file formats are identical */
 			unsigned char inbuf[4096];
 			size_t nread, nwritten;
 			int fd0, fd1;
 			ssize_t n;
 			
-			filename = g_build_filename (old_path, dent->d_name, NULL);
-			if ((fd0 = open (filename, O_RDONLY)) == -1) {
+			filename = g_build_filename (old_path, dent, NULL);
+			if ((fd0 = g_open (filename, O_RDONLY|O_BINARY, 0)) == -1) {
 				g_free (filename);
 				continue;
 			}
 			
 			g_free (filename);
-			filename = g_build_filename (new_path, dent->d_name, NULL);
-			if ((fd1 = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0666)) == -1) {
+			filename = g_build_filename (new_path, dent, NULL);
+			if ((fd1 = g_open (filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666)) == -1) {
 				g_free (filename);
 				close (fd0);
 				continue;
@@ -665,22 +679,22 @@ migrate_pilot_data (const char *component, const char *conduit, const char *old_
 				n = fsync (fd1);
 			
 			if (n == -1) {
-				g_warning ("Failed to migrate %s: %s", dent->d_name, strerror (errno));
-				unlink (filename);
+				g_warning ("Failed to migrate %s: %s", dent, strerror (errno));
+				g_unlink (filename);
 			}
 			
 			close (fd0);
 			close (fd1);
 			g_free (filename);
-		} else if (!strncmp (dent->d_name, changelog, strlen (changelog)) &&
-			   ((ext = strrchr (dent->d_name, '.')) && !strcmp (ext, ".db"))) {
+		} else if (!strncmp (dent, changelog, strlen (changelog)) &&
+			   ((ext = strrchr (dent, '.')) && !strcmp (ext, ".db"))) {
 			/* src and dest formats differ, src format is db3 while dest format is xml */
 			EXmlHash *xmlhash;
 			EDbHash *dbhash;
 			struct stat st;
 			
-			filename = g_build_filename (old_path, dent->d_name, NULL);
-			if (stat (filename, &st) == -1) {
+			filename = g_build_filename (old_path, dent, NULL);
+			if (g_stat (filename, &st) == -1) {
 				g_free (filename);
 				continue;
 			}
@@ -688,9 +702,9 @@ migrate_pilot_data (const char *component, const char *conduit, const char *old_
 			dbhash = e_dbhash_new (filename);
 			g_free (filename);
 			
-			filename = g_strdup_printf ("%s/%s.ics-%s", new_path, component, dent->d_name);
-			if (stat (filename, &st) != -1)
-				unlink (filename);
+			filename = g_strdup_printf ("%s/%s.ics-%s", new_path, component, dent);
+			if (g_stat (filename, &st) != -1)
+				g_unlink (filename);
 			xmlhash = e_xmlhash_new (filename);
 			g_free (filename);
 			
@@ -703,9 +717,11 @@ migrate_pilot_data (const char *component, const char *conduit, const char *old_
 		}
 	}
 	
-	closedir (dir);
+	g_dir_close (dir);
 }
 
+#endif
+
 gboolean
 migrate_calendars (CalendarComponent *component, int major, int minor, int revision, GError **err)
 {
@@ -720,6 +736,7 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 	   groups/sources. */
 	create_calendar_sources (component, calendar_component_peek_source_list (component), &on_this_computer, &personal_source, &on_the_web, &contacts);
 
+#ifndef G_OS_WIN32
 	if (major == 1) {
 		xmlDocPtr config_doc = NULL;
 		char *conf_file;
@@ -844,6 +861,7 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 		}
 
 	}
+#endif	/* !G_OS_WIN32 */
 
 	e_source_list_sync (calendar_component_peek_source_list (component), NULL);
 
@@ -888,14 +906,14 @@ migrate_tasks (TasksComponent *component, int major, int minor, int revision, GE
 	   groups/sources. */
 	create_task_sources (component, tasks_component_peek_source_list (component), &on_this_computer, &on_the_web, &personal_source);
 	
+#ifndef G_OS_WIN32
 	if (major == 1) {
 		xmlDocPtr config_doc = NULL;
 		char *conf_file;
-		struct stat st;
 
 		conf_file = g_build_filename (g_get_home_dir (), "evolution", "config.xmldb", NULL);
-		if (lstat (conf_file, &st) == 0 && S_ISREG (st.st_mode)) 
-			config_doc = xmlParseFile (conf_file);
+		if (g_file_test (conf_file, G_FILE_TEST_IS_REGULAR))
+			config_doc = e_xml_parse_file (conf_file);
 		g_free (conf_file);
 		
 		if (config_doc && minor <= 2) {
@@ -984,7 +1002,7 @@ migrate_tasks (TasksComponent *component, int major, int minor, int revision, GE
 			}
 		}
 	}
-
+#endif	/* !G_OS_WIN32 */
 	e_source_list_sync (tasks_component_peek_source_list (component), NULL);
 	retval = TRUE;
 fail:
@@ -997,3 +1015,123 @@ fail:
 	
         return retval;
 }
+
+/********************************************************************************************************
+ *
+ * 		MEMOS
+ * 
+ ********************************************************************************************************/
+
+static void
+create_memo_sources (MemosComponent *component,
+		     ESourceList   *source_list,
+		     ESourceGroup **on_this_computer,
+		     ESourceGroup **on_the_web,
+		     ESource **personal_source)
+{
+	GSList *groups;
+	ESourceGroup *group;
+	char *base_uri, *base_uri_proto;
+
+	*on_this_computer = NULL;
+	*on_the_web = NULL;
+	*personal_source = NULL;
+	
+	base_uri = g_build_filename (memos_component_peek_base_directory (component),
+				     "memos", "local", NULL);
+
+	base_uri_proto = g_filename_to_uri (base_uri, NULL, NULL);
+
+	groups = e_source_list_peek_groups (source_list);
+	if (groups) {
+		/* groups are already there, we need to search for things... */
+		GSList *g;
+
+		for (g = groups; g; g = g->next) {
+
+			group = E_SOURCE_GROUP (g->data);
+
+			if (!*on_this_computer && !strcmp (base_uri_proto, e_source_group_peek_base_uri (group)))
+				*on_this_computer = g_object_ref (group);
+			else if (!*on_the_web && !strcmp (WEBCAL_BASE_URI, e_source_group_peek_base_uri (group)))
+				*on_the_web = g_object_ref (group);
+		}
+	}
+
+	if (*on_this_computer) {
+		/* make sure "Personal" shows up as a source under
+		   this group */
+		GSList *sources = e_source_group_peek_sources (*on_this_computer);
+		GSList *s;
+		for (s = sources; s; s = s->next) {
+			ESource *source = E_SOURCE (s->data);
+			if (!strcmp (PERSONAL_RELATIVE_URI, e_source_peek_relative_uri (source))) {
+				*personal_source = g_object_ref (source);
+				break;
+			}
+		}
+	} else {
+		/* create the local source group */
+		group = e_source_group_new (_("On This Computer"), base_uri_proto);
+		e_source_list_add_group (source_list, group, -1);
+
+		*on_this_computer = group;
+	}
+
+	if (!*personal_source) {
+		/* Create the default Person task list */
+		ESource *source = e_source_new (_("Personal"), PERSONAL_RELATIVE_URI);
+		e_source_group_add_source (*on_this_computer, source, -1);
+
+		if (!calendar_config_get_primary_memos () && !calendar_config_get_memos_selected ()) {
+			GSList selected;
+
+			calendar_config_set_primary_memos (e_source_peek_uid (source));
+
+			selected.data = (gpointer)e_source_peek_uid (source);
+			selected.next = NULL;
+			calendar_config_set_memos_selected (&selected);
+		}
+
+		e_source_set_color (source, 0xBECEDD);
+		*personal_source = source;
+	}
+
+	if (!*on_the_web) {
+		/* Create the Webcal source group */
+		group = e_source_group_new (_("On The Web"), WEBCAL_BASE_URI);
+		e_source_list_add_group (source_list, group, -1);
+
+		*on_the_web = group;
+	}
+
+	g_free (base_uri_proto);
+	g_free (base_uri);
+}
+
+
+gboolean
+migrate_memos (MemosComponent *component, int major, int minor, int revision, struct _GError **err)
+{
+	ESourceGroup *on_this_computer = NULL;
+	ESourceGroup *on_the_web = NULL;
+	ESource *personal_source = NULL;
+	gboolean retval = FALSE;
+
+	/* we call this unconditionally now - create_groups either
+	   creates the groups/sources or it finds the necessary
+	   groups/sources. */
+	create_memo_sources (component, memos_component_peek_source_list (component), &on_this_computer, &on_the_web, &personal_source);
+
+	e_source_list_sync (memos_component_peek_source_list (component), NULL);
+	retval = TRUE;
+fail:
+	if (on_this_computer)
+		g_object_unref (on_this_computer);
+	if (on_the_web)
+		g_object_unref (on_the_web);
+	if (personal_source)
+		g_object_unref (personal_source);
+	
+        return retval;
+}
