@@ -31,8 +31,10 @@
 #include <bonobo/bonobo-exception.h>
 #include <gconf/gconf-client.h>
 #include <libecal/e-cal.h>
+#include <libedataserver/e-data-server-util.h>
 #include <libedataserverui/e-source-selector.h>
 #include <shell/e-user-creatable-items-handler.h>
+#include <shell/e-component-view.h>
 #include "e-cal-model.h"
 #include "e-tasks.h"
 #include "tasks-component.h"
@@ -1124,16 +1126,15 @@ view_destroyed_cb (gpointer data, GObject *where_the_object_was)
 	}
 }
 
-static void
-impl_createControls (PortableServer_Servant servant,
-		     Bonobo_Control *corba_sidebar_control,
-		     Bonobo_Control *corba_view_control,
-		     Bonobo_Control *corba_statusbar_control,
-		     CORBA_Environment *ev)
+static GNOME_Evolution_ComponentView
+impl_createView (PortableServer_Servant servant,
+		 GNOME_Evolution_ShellView parent,
+		 CORBA_Environment *ev)
 {
 	TasksComponent *component = TASKS_COMPONENT (bonobo_object_from_servant (servant));
 	TasksComponentPrivate *priv;
 	TasksComponentView *component_view;
+	EComponentView *ecv;
 	
 	priv = component->priv;
 
@@ -1143,16 +1144,17 @@ impl_createControls (PortableServer_Servant servant,
 		/* FIXME Should we describe the problem in a control? */
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
 
-		return;
+		return CORBA_OBJECT_NIL;
 	}
 
 	g_object_weak_ref (G_OBJECT (component_view->view_control), view_destroyed_cb, component);
 	priv->views = g_list_append (priv->views, component_view);
 	
-	/* Return the controls */
-	*corba_sidebar_control = CORBA_Object_duplicate (BONOBO_OBJREF (component_view->sidebar_control), ev);
-	*corba_view_control = CORBA_Object_duplicate (BONOBO_OBJREF (component_view->view_control), ev);
-	*corba_statusbar_control = CORBA_Object_duplicate (BONOBO_OBJREF (component_view->statusbar_control), ev);
+	/* TODO: Make TasksComponentView just subclass EComponentView */
+	ecv = e_component_view_new_controls (parent, "tasks", component_view->sidebar_control,
+					     component_view->view_control, component_view->statusbar_control);
+
+	return BONOBO_OBJREF(ecv);
 }
 
 static GNOME_Evolution_CreatableItemTypeList *
@@ -1179,7 +1181,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 	list->_buffer[1].description = _("New assigned task");
 	list->_buffer[1].menuDescription = _("Assigne_d Task");
 	list->_buffer[1].tooltip = _("Create a new assigned task");
-	list->_buffer[1].menuShortcut = 'd';
+	list->_buffer[1].menuShortcut = '\0';
 	list->_buffer[1].iconName = "stock_task";
 	list->_buffer[1].type = GNOME_Evolution_CREATABLE_OBJECT;
 
@@ -1187,7 +1189,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 	list->_buffer[2].description = _("New task list");
 	list->_buffer[2].menuDescription = _("Task l_ist");
 	list->_buffer[2].tooltip = _("Create a new task list");
-	list->_buffer[2].menuShortcut = 'i';
+	list->_buffer[2].menuShortcut = '\0';
 	list->_buffer[2].iconName = "stock_todo";
 	list->_buffer[2].type = GNOME_Evolution_CREATABLE_FOLDER;
 
@@ -1286,7 +1288,7 @@ tasks_component_class_init (TasksComponentClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	epv->upgradeFromVersion      = impl_upgradeFromVersion;
-	epv->createControls          = impl_createControls;
+	epv->createView		     = impl_createView;
 	epv->_get_userCreatableItems = impl__get_userCreatableItems;
 	epv->requestCreateItem       = impl_requestCreateItem;
 
@@ -1320,7 +1322,7 @@ tasks_component_peek (void)
 	if (component == NULL) {
 		component = g_object_new (tasks_component_get_type (), NULL);
 
-		if (e_mkdir_hier (component->priv->config_directory, 0777) != 0) {
+		if (e_util_mkdir_hier (component->priv->config_directory, 0777) != 0) {
 			g_warning (G_STRLOC ": Cannot create directory %s: %s",
 				   component->priv->config_directory, g_strerror (errno));
 			g_object_unref (component);
