@@ -37,6 +37,7 @@
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libedataserverui/e-name-selector.h>
+#include <libedataserverui/e-name-selector-entry.h>
 
 #include "Composer.h"
 
@@ -56,8 +57,11 @@
 #include "e-util/e-error.h"
 
 #include <camel/camel.h>
+#include <camel/camel-store.h>
+#include <camel/camel-session.h>
 #include "e-msg-composer-hdrs.h"
 #include "mail/mail-config.h"
+#include "mail/mail-session.h"
 /*#include "mail/em-folder-selection-button.h"*/
 #include "mail/em-folder-selector.h"
 
@@ -88,7 +92,7 @@ extern struct _EMFolderTreeModel *mail_component_peek_tree_model(struct _MailCom
 typedef struct {
 	GtkWidget *label;
 	GtkWidget *entry;
-	int visible:1;
+	guint visible:1;
 } EMsgComposerHdrPair;
 
 struct _EMsgComposerHdrsPrivate {
@@ -126,8 +130,6 @@ static int signals[LAST_SIGNAL];
 static void
 addressbook_dialog_response (ENameSelectorDialog *name_selector_dialog, gint response, gpointer user_data)
 {
-	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (user_data);
-
 	gtk_widget_hide (GTK_WIDGET (name_selector_dialog));
 }
 
@@ -186,6 +188,13 @@ address_button_clicked_cb (GtkButton *button, gpointer data)
 	emchas = data;
 	hdrs = emchas->hdrs;
 	priv = hdrs->priv;
+	
+	if(button == (GtkButton *)hdrs->priv->to.label)
+		gtk_widget_grab_focus(hdrs->priv->to.entry);
+	else if(button == (GtkButton *)priv->cc.label)
+		gtk_widget_grab_focus(hdrs->priv->cc.entry);
+	else if(button == (GtkButton *)priv->bcc.label)
+		gtk_widget_grab_focus(hdrs->priv->bcc.entry);
 
 	name_selector_dialog = e_name_selector_peek_dialog (priv->name_selector);
 	gtk_widget_show (GTK_WIDGET (name_selector_dialog));
@@ -219,6 +228,26 @@ from_changed (GtkWidget *item, gpointer data)
 	g_signal_emit (hdrs, signals [FROM_CHANGED], 0);
 }
 
+static gboolean
+account_can_send (EAccount *account)
+{
+	static CamelStore *store;
+	CamelException ex;
+	gboolean result = FALSE;
+	
+	if (!account->parent_uid) 
+		return TRUE;
+		 
+       	if (!(store = (CamelStore *) camel_session_get_service (session, e_account_get_string(account, E_ACCOUNT_SOURCE_URL), CAMEL_PROVIDER_STORE, &ex))) {	  
+		camel_exception_clear (&ex);
+		return result;
+	} else if (store->mode & CAMEL_STORE_WRITE)
+		result = TRUE;
+
+	camel_object_unref (store);
+	return result;
+}
+
 static void
 account_added_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs *hdrs)
 {
@@ -228,22 +257,24 @@ account_added_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs *h
 	omenu = e_msg_composer_hdrs_get_from_omenu (hdrs);
 	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
 	
-	label = g_strdup_printf ("%s <%s>", account->id->name, account->id->address);
-	item = gtk_menu_item_new_with_label (label);
-	gtk_widget_show (item);
-	g_free (label);
+	if (account_can_send (account)) {
+		label = g_strdup_printf ("%s <%s>", account->id->name, account->id->address);
+		item = gtk_menu_item_new_with_label (label);
+		gtk_widget_show (item);
+		g_free (label);
+
+		g_object_ref (account);
+		g_object_set_data ((GObject *) item, "account", account);
+		g_signal_connect (item, "activate", G_CALLBACK (from_changed), hdrs);
 	
-	g_object_ref (account);
-	g_object_set_data ((GObject *) item, "account", account);
-	g_signal_connect (item, "activate", G_CALLBACK (from_changed), hdrs);
+		/* this is so we can later set which one we want */
+		hdrs->priv->from_options = g_slist_append (hdrs->priv->from_options, item);
 	
-	/* this is so we can later set which one we want */
-	hdrs->priv->from_options = g_slist_append (hdrs->priv->from_options, item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	
-	toplevel = gtk_widget_get_toplevel ((GtkWidget *) hdrs);
-	gtk_widget_set_sensitive (toplevel, TRUE);
+		toplevel = gtk_widget_get_toplevel ((GtkWidget *) hdrs);
+		gtk_widget_set_sensitive (toplevel, TRUE);
+	}
 }
 
 static void
@@ -253,7 +284,7 @@ account_changed_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs
 	EAccount *acnt;
 	GSList *node;
 	char *text;
-	
+
 	node = hdrs->priv->from_options;
 	while (node != NULL) {
 		item = node->data;
@@ -277,7 +308,7 @@ account_removed_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs
 	GtkWidget *item, *omenu, *toplevel;
 	EAccount *acnt;
 	GSList *node;
-	
+
 	node = priv->from_options;
 	while (node != NULL) {
 		item = node->data;
@@ -286,8 +317,7 @@ account_removed_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs
 			if (hdrs->account == account)
 				hdrs->account = NULL;
 			
-			priv->from_options = g_slist_remove_link (priv->from_options, node);
-			g_slist_free_1 (node);
+			priv->from_options = g_slist_delete_link (priv->from_options, node);
 			g_object_unref (account);
 			gtk_widget_destroy (item);
 			break;
@@ -327,7 +357,7 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 	EAccount *account;
 	EIterator *iter;
 	char *uid;
-	
+
 	omenu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
 	
@@ -340,8 +370,8 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 	iter = e_list_get_iterator ((EList *) priv->accounts);
 	while (e_iterator_is_valid (iter)) {
 		account = (EAccount *) e_iterator_get (iter);
-		
-		if (account->id->address)
+
+		if (account->id->address && account_can_send (account))
 			g_ptr_array_add (addresses, account->id->address);
 		
 		e_iterator_next (iter);
@@ -360,7 +390,7 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 			continue;
 		}
 		
-		if (account->id->address && *account->id->address) {
+		if (account->id->address && *account->id->address && account_can_send (account)) {
 			/* If the account has a unique email address, just
 			 * show that. Otherwise include the account name.
 			 */
@@ -499,6 +529,7 @@ post_browser_response (EMFolderSelector *emfs, int response, EMsgComposerHdrs *h
 {
 	if (response == GTK_RESPONSE_OK) {
 		GList *uris = em_folder_selector_get_selected_uris (emfs);
+
 		e_msg_composer_hdrs_set_post_to_list (hdrs, uris);
 		hdrs->priv->post_custom = FALSE;
 		g_list_foreach (uris, (GFunc) g_free, NULL);
@@ -516,6 +547,7 @@ post_browser_clicked_cb (GtkButton *button, EMsgComposerHdrs *hdrs)
 	GtkWidget *dialog;
 	GList *post_items;
 	
+	gtk_widget_grab_focus(hdrs->priv->post_to.entry);
 	model = mail_component_peek_tree_model (mail_component_peek ());
 	emft = (EMFolderTree *) em_folder_tree_new_with_model (model);
 	em_folder_tree_set_multiselect (emft, TRUE);
@@ -543,11 +575,8 @@ post_entry_changed_cb (GtkButton *button, EMsgComposerHdrs *hdrs)
 static EMsgComposerHdrPair 
 header_new_recipient (EMsgComposerHdrs *hdrs, const char *name, const char *tip)
 {
-	EMsgComposerHdrsPrivate *priv;
 	EMsgComposerHdrPair ret;
 	
-	priv = hdrs->priv;
-	
 	ret.label = gtk_button_new_with_mnemonic (name);
 	GTK_OBJECT_UNSET_FLAGS (ret.label, GTK_CAN_FOCUS);
 	g_signal_connect_data (ret.label, "clicked",
@@ -569,7 +598,7 @@ static void
 entry_changed (GtkWidget *entry, EMsgComposerHdrs *hdrs)
 {
 	const char *subject;
-	
+
 	subject = e_msg_composer_hdrs_get_subject (hdrs);
 	g_signal_emit (hdrs, signals[SUBJECT_CHANGED], 0, subject);
 	g_signal_emit (hdrs, signals[HDRS_CHANGED], 0);
@@ -589,23 +618,22 @@ create_headers (EMsgComposerHdrs *hdrs)
 	 */
 	priv->reply_to.label = gtk_label_new_with_mnemonic (_("_Reply-To:"));
 	priv->reply_to.entry = gtk_entry_new ();
-	gtk_label_set_mnemonic_widget (priv->reply_to.label, priv->reply_to.entry);
+	gtk_label_set_mnemonic_widget((GtkLabel *)priv->reply_to.label, priv->reply_to.entry);
 	
 	/*
 	 * From
 	 */
 	priv->from.label = gtk_label_new_with_mnemonic (_("Fr_om:"));
 	priv->from.entry = create_from_optionmenu (hdrs);
-	gtk_label_set_mnemonic_widget (priv->from.label, e_msg_composer_hdrs_get_from_omenu (hdrs));
+	gtk_label_set_mnemonic_widget((GtkLabel *)priv->from.label, e_msg_composer_hdrs_get_from_omenu (hdrs));
 	
 	/*
 	 * Subject
 	 */
 	priv->subject.label = gtk_label_new_with_mnemonic (_("S_ubject:"));
 	priv->subject.entry = gtk_entry_new ();
-	gtk_label_set_mnemonic_widget (priv->subject.label, priv->subject.entry);
-	g_signal_connect (priv->subject.entry, "changed",
-			  G_CALLBACK (entry_changed), hdrs);
+	gtk_label_set_mnemonic_widget((GtkLabel *)priv->subject.label, priv->subject.entry);
+	g_signal_connect(priv->subject.entry, "changed", G_CALLBACK(entry_changed), hdrs);
 
 	/*
 	 * To, CC, and Bcc
@@ -1090,7 +1118,7 @@ e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	
 	omenu = GTK_OPTION_MENU (e_msg_composer_hdrs_get_from_omenu (hdrs));
-	
+
 	if (!account_name) {
 		gconf = gconf_client_get_default ();
 		uid = gconf_client_get_string (gconf, "/apps/evolution/mail/default_account", NULL);
@@ -1104,24 +1132,26 @@ e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
 		item = l->data;
 		
 		account = g_object_get_data ((GObject *) item, "account");
-		if (account_name) {
-			if (account->name && !strcmp (account_name, account->name)) {
-				/* set the correct optionlist item */
+		if (account_can_send (account))
+		{
+			if (account_name) {
+				if (account->name && !strcmp (account_name, account->name)) {
+					/* set the correct optionlist item */
+					gtk_option_menu_set_history (omenu, i);
+					g_signal_emit_by_name (item, "activate", hdrs);
+					g_free (uid);
+					
+					return;
+				}
+			} else if (uid && !strcmp (account->uid, uid)) {
+				/* set the default optionlist item */
 				gtk_option_menu_set_history (omenu, i);
 				g_signal_emit_by_name (item, "activate", hdrs);
 				g_free (uid);
 				
 				return;
 			}
-		} else if (uid && !strcmp (account->uid, uid)) {
-			/* set the default optionlist item */
-			gtk_option_menu_set_history (omenu, i);
-			g_signal_emit_by_name (item, "activate", hdrs);
-			g_free (uid);
-			
-			return;
 		}
-		
 		l = l->next;
 		i++;
 	}
@@ -1171,8 +1201,6 @@ void
 e_msg_composer_hdrs_set_to (EMsgComposerHdrs *hdrs,
 			    EDestination **to_destv)
 {
-	char *str;
-	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 
 	destinations_to_name_selector_entry (E_NAME_SELECTOR_ENTRY (hdrs->priv->to.entry), to_destv);
@@ -1182,8 +1210,6 @@ void
 e_msg_composer_hdrs_set_cc (EMsgComposerHdrs *hdrs,
 			    EDestination **cc_destv)
 {
-	char *str;
-	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 
 	destinations_to_name_selector_entry (E_NAME_SELECTOR_ENTRY (hdrs->priv->cc.entry), cc_destv);
@@ -1196,8 +1222,6 @@ void
 e_msg_composer_hdrs_set_bcc (EMsgComposerHdrs *hdrs,
 			     EDestination **bcc_destv)
 {
-	char *str;
-	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 
 	destinations_to_name_selector_entry (E_NAME_SELECTOR_ENTRY (hdrs->priv->bcc.entry), bcc_destv);
@@ -1253,8 +1277,10 @@ get_account_store_url (EMsgComposerHdrs *hdrs)
 	CamelURL *url;
 	char *ret = NULL;
 	
-	if (hdrs->account->source && hdrs->account->source->url) {
-		url = camel_url_new (hdrs->account->source->url, NULL);
+	if (hdrs->account->source
+	    && hdrs->account->source->url
+	    && hdrs->account->source->url[0]
+	    && (url = camel_url_new (hdrs->account->source->url, NULL))) {
 		ret = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
 		camel_url_free (url);
 	}
@@ -1362,7 +1388,7 @@ e_msg_composer_hdrs_get_from (EMsgComposerHdrs *hdrs)
 {
 	CamelInternetAddress *addr;
 	EAccount *account;
-	
+
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
 	if (!(account = hdrs->account)) {
@@ -1423,7 +1449,6 @@ destination_list_to_destv (GList *destinations)
 EDestination **
 e_msg_composer_hdrs_get_to (EMsgComposerHdrs *hdrs)
 {
-	char *str = NULL;
 	EDestinationStore *destination_store;
 	GList *destinations;
 	EDestination **destv = NULL;
@@ -1443,7 +1468,6 @@ e_msg_composer_hdrs_get_to (EMsgComposerHdrs *hdrs)
 EDestination **
 e_msg_composer_hdrs_get_cc (EMsgComposerHdrs *hdrs)
 {
-	char *str = NULL;
 	EDestinationStore *destination_store;
 	GList *destinations;
 	EDestination **destv = NULL;
@@ -1463,7 +1487,6 @@ e_msg_composer_hdrs_get_cc (EMsgComposerHdrs *hdrs)
 EDestination **
 e_msg_composer_hdrs_get_bcc (EMsgComposerHdrs *hdrs)
 {
-	char *str = NULL;
 	EDestinationStore *destination_store;
 	GList *destinations;
 	EDestination **destv = NULL;
@@ -1636,3 +1659,4 @@ e_msg_composer_hdrs_get_from_omenu (EMsgComposerHdrs *hdrs)
 	
 	return GTK_WIDGET (g_object_get_data ((GObject *) hdrs->priv->from.entry, "from_menu"));
 }
+
