@@ -108,16 +108,13 @@ static void
 emft_copy_folders__copy (struct _mail_msg *mm)
 {
 	struct _EMCopyFolders *m = (struct _EMCopyFolders *) mm;
-	guint32 flags = CAMEL_STORE_FOLDER_INFO_FAST | CAMEL_STORE_FOLDER_INFO_RECURSIVE;
+	guint32 flags = CAMEL_STORE_FOLDER_INFO_FAST | CAMEL_STORE_FOLDER_INFO_RECURSIVE | CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
 	GList *pending = NULL, *deleting = NULL, *l;
 	GString *fromname, *toname;
 	CamelFolderInfo *fi;
 	const char *tmp;
 	int fromlen;
 	
-	if (camel_store_supports_subscriptions (m->fromstore))
-		flags |= CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
-	
 	if (!(fi = camel_store_get_folder_info (m->fromstore, m->frombase, flags, &mm->ex)))
 		return;
 	
@@ -247,11 +244,12 @@ static struct _mail_msg_op copy_folders_op = {
 	emft_copy_folders__free,
 };
 
-static void
-emfu_copy_folders (CamelStore *tostore, const char *tobase, CamelStore *fromstore, const char *frombase, int delete)
+int
+em_folder_utils_copy_folders(CamelStore *fromstore, const char *frombase, CamelStore *tostore, const char *tobase, int delete)
 {
 	struct _EMCopyFolders *m;
-	
+	int seq;
+
 	m = mail_msg_new (&copy_folders_op, NULL, sizeof (struct _EMCopyFolders));
 	camel_object_ref (fromstore);
 	m->fromstore = fromstore;
@@ -260,8 +258,11 @@ emfu_copy_folders (CamelStore *tostore, const char *tobase, CamelStore *fromstor
 	m->frombase = g_strdup (frombase);
 	m->tobase = g_strdup (tobase);
 	m->delete = delete;
-	
+	seq = m->msg.seq;
+
 	e_thread_put (mail_thread_new, (EMsg *) m);
+
+	return seq;
 }
 
 struct _copy_folder_data {
@@ -311,7 +312,7 @@ emfu_copy_folder_selected (const char *uri, void *data)
 	if (tobase == NULL)
 		tobase = "";
 
-	emfu_copy_folders (tostore, tobase, fromstore, cfd->fi->full_name, cfd->delete);
+	em_folder_utils_copy_folders(fromstore, cfd->fi->full_name, tostore, tobase, cfd->delete);
 	
 	camel_url_free (url);
 fail:
@@ -356,31 +357,15 @@ emfu_copy_folder_exclude(EMFolderTree *tree, GtkTreeModel *model, GtkTreeIter *i
 /* FIXME: this interface references the folderinfo without copying it  */
 /* FIXME: these functions must be documented */
 void
-em_folder_utils_copy_folder (CamelFolderInfo *folderinfo)
+em_folder_utils_copy_folder(CamelFolderInfo *folderinfo, int delete)
 {
 	struct _copy_folder_data *cfd;
 	
 	cfd = g_malloc (sizeof (*cfd));
 	cfd->fi = folderinfo;
-	cfd->delete = FALSE;
+	cfd->delete = delete;
 
-	em_select_folder (NULL, _("Select folder"), _("C_opy"),
-			  NULL, emfu_copy_folder_exclude,
-			  emfu_copy_folder_selected, cfd);}
-
-
-/* FIXME: this interface references the folderinfo without copying it  */
-/* FIXME: these functions must be documented */
-void
-em_folder_utils_move_folder (CamelFolderInfo *folderinfo)
-{
-	struct _copy_folder_data *cfd;
-
-	cfd = g_malloc (sizeof (*cfd));
-	cfd->fi = folderinfo;
-	cfd->delete = TRUE;
-
-	em_select_folder (NULL, _("Select folder"), _("_Move"),
+	em_select_folder (NULL, _("Select folder"), delete?_("_Move"):_("C_opy"),
 			  NULL, emfu_copy_folder_exclude,
 			  emfu_copy_folder_selected, cfd);
 }
@@ -431,12 +416,9 @@ emfu_delete_rec (CamelStore *store, CamelFolderInfo *fi, CamelException *ex)
 static void
 emfu_delete_folders (CamelStore *store, const char *full_name, CamelException *ex)
 {
-	guint32 flags = CAMEL_STORE_FOLDER_INFO_RECURSIVE | CAMEL_STORE_FOLDER_INFO_FAST;
+	guint32 flags = CAMEL_STORE_FOLDER_INFO_RECURSIVE | CAMEL_STORE_FOLDER_INFO_FAST | CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
 	CamelFolderInfo *fi;
 	
-	if (camel_store_supports_subscriptions (store))
-		flags |= CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
-	
 	fi = camel_store_get_folder_info (store, full_name, flags, ex);
 	if (camel_exception_is_set (ex))
 		return;
@@ -487,7 +469,7 @@ em_folder_utils_delete_folder (CamelFolder *folder)
 	dialog = e_error_new(NULL,
 			     (folder->parent_store && CAMEL_IS_VEE_STORE(folder->parent_store))?"mail:ask-delete-vfolder":"mail:ask-delete-folder",
 			     folder->full_name, NULL);
-	g_object_set_data_full ((GObject *) dialog, "full_name", folder->full_name, g_free);
+	g_object_set_data_full ((GObject *) dialog, "full_name", g_strdup (folder->full_name), g_free);
 	g_object_set_data_full ((GObject *) dialog, "store", folder->parent_store, camel_object_unref);
 	g_signal_connect (dialog, "response", G_CALLBACK (emfu_delete_response), NULL);
 	gtk_widget_show (dialog);
