@@ -26,6 +26,18 @@
 
 #include <string.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <gdk/gdkkeysyms.h>
+
+#ifdef G_OS_WIN32
+/* Work around 'DATADIR' and 'interface' lossage in <windows.h> */
+#define DATADIR crap_DATADIR
+#include <windows.h>
+#undef DATADIR
+#undef interface
+#endif
+
 #include <gtkhtml/gtkhtml.h>
 #include <gtkhtml/gtkhtml-embedded.h>
 #include <gtkhtml/gtkhtml-search.h>
@@ -68,8 +80,9 @@
 #include <camel/camel-folder.h>
 #include <camel/camel-string-utils.h>
 
-/* should this be in e-util rather than gal? */
+#include <misc/e-cursors.h>
 #include <e-util/e-util.h>
+#include <e-util/e-util-private.h>
 
 #include <libedataserver/e-msgport.h>
 #include <e-util/e-gui-utils.h>
@@ -120,6 +133,7 @@ struct _EMFormatHTMLDisplayPrivate {
 	GtkWidget *attachment_bar;
 	GtkWidget *attachment_box;
 	GtkWidget *label;
+	GtkWidget *save_txt;
 	GtkWidget *arrow;
 	GtkWidget *forward;
 	GtkWidget *down;
@@ -388,6 +402,24 @@ em_format_html_display_get_type(void)
 	return type;
 }
 
+static gboolean
+efhd_scroll_event(GtkWidget *w, GdkEventScroll *event, EMFormatHTMLDisplay *efhd)
+{
+	if(event->state & GDK_CONTROL_MASK)
+	{
+		if(event->direction == GDK_SCROLL_UP)
+		{
+			gtk_html_zoom_in (efhd->formathtml.html);
+		}
+		else if(event->direction == GDK_SCROLL_DOWN)
+		{
+			gtk_html_zoom_out (efhd->formathtml.html);	
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 EMFormatHTMLDisplay *em_format_html_display_new(void)
 {
 	EMFormatHTMLDisplay *efhd;
@@ -398,6 +430,7 @@ EMFormatHTMLDisplay *em_format_html_display_new(void)
 	g_signal_connect(efhd->formathtml.html, "link_clicked", G_CALLBACK(efhd_html_link_clicked), efhd);
 	g_signal_connect(efhd->formathtml.html, "on_url", G_CALLBACK(efhd_html_on_url), efhd);
 	g_signal_connect(efhd->formathtml.html, "button_press_event", G_CALLBACK(efhd_html_button_press_event), efhd);
+	g_signal_connect(efhd->formathtml.html,"scroll_event", G_CALLBACK(efhd_scroll_event), efhd);
 
 	return efhd;
 }
@@ -465,6 +498,8 @@ efhd_update_matches(EMFormatHTMLDisplay *efhd)
 		sprintf(str, fmt, e_searching_tokenizer_match_count(efhd->search_tok));
 		gtk_label_set_text((GtkLabel *)p->search_matches_label, str);
 	}
+	gtk_widget_show((GtkWidget *)p->search_matches_label);
+	
 }
 
 static void
@@ -545,13 +580,19 @@ em_format_html_display_search(EMFormatHTMLDisplay *efhd)
 {
 	struct _EMFormatHTMLDisplayPrivate *p = efhd->priv;
 	GladeXML *xml;
+	char *gladefile;
 
 	if (p->search_dialog) {
 		gdk_window_raise(((GtkWidget *)p->search_dialog)->window);
 		return;
 	}
 
-	xml = glade_xml_new (EVOLUTION_GLADEDIR "/mail-dialogs.glade", "search_message_dialog", NULL);
+	gladefile = g_build_filename (EVOLUTION_GLADEDIR,
+				      "mail-dialogs.glade",
+				      NULL);
+	xml = glade_xml_new (gladefile, "search_message_dialog", NULL);
+	g_free (gladefile);
+
 	if (xml == NULL) {
 		g_warning("Cannot open search dialog glade file");
 		/* ?? */
@@ -576,6 +617,7 @@ em_format_html_display_search(EMFormatHTMLDisplay *efhd)
 	g_signal_connect(p->search_case_check, "toggled", G_CALLBACK(efhd_search_case_toggled), efhd);
 	g_signal_connect(p->search_dialog, "response", G_CALLBACK(efhd_search_response), efhd);
 	gtk_widget_show((GtkWidget *)p->search_dialog);
+	gtk_widget_hide((GtkWidget *)p->search_matches_label);
 }
 
 void
@@ -718,10 +760,10 @@ static const struct {
 static const struct {
 	const char *icon, *shortdesc, *description;
 } smime_encrypt_table[4] = {
-	{ "stock_lock-broken", N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across the Internet.") },
-	{ "stock_lock-ok", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm.  It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
+	{ "stock_lock-broken", N_("Unencrypted"), N_("This message is not encrypted. Its content may be viewed in transit across the Internet.") },
+	{ "stock_lock-ok", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm. It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
 	{ "stock_lock-ok", N_("Encrypted"), N_("This message is encrypted.  It would be difficult for an outsider to view the content of this message.") },
-	{ "stock_lock-ok", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm.  It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
+	{ "stock_lock-ok", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm. It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
 };
 
 static const char *smime_sign_colour[4] = {
@@ -855,12 +897,18 @@ efhd_xpkcs7mime_validity_clicked(GtkWidget *button, EMFormatHTMLPObject *pobject
 	struct _smime_pobject *po = (struct _smime_pobject *)pobject;
 	GladeXML *xml;
 	GtkWidget *vbox, *w;
+	char *gladefile;
 
 	if (po->widget)
 		/* FIXME: window raise? */
 		return;
 
-	xml = glade_xml_new(EVOLUTION_GLADEDIR "/mail-dialogs.glade", "message_security_dialog", NULL);
+	gladefile = g_build_filename (EVOLUTION_GLADEDIR,
+				      "mail-dialogs.glade",
+				      NULL);
+	xml = glade_xml_new(gladefile, "message_security_dialog", NULL);
+	g_free (gladefile);
+
 	po->widget = glade_xml_get_widget(xml, "message_security_dialog");
 
 	vbox = glade_xml_get_widget(xml, "signature_vbox");
@@ -1285,7 +1333,7 @@ static void
 efhd_drag_data_get(GtkWidget *w, GdkDragContext *drag, GtkSelectionData *data, guint info, guint time, EMFormatHTMLPObject *pobject)
 {
 	CamelMimePart *part = pobject->part;
-	char *uri, *path;
+	char *uri, *uri_crlf, *path;
 	CamelStream *stream;
 
 	switch (info) {
@@ -1319,10 +1367,12 @@ efhd_drag_data_get(GtkWidget *w, GdkDragContext *drag, GtkSelectionData *data, g
 		if (path == NULL)
 			return;
 
-		uri = g_strdup_printf("file://%s\r\n", path);
+		uri = g_filename_to_uri(path, NULL, NULL);
 		g_free(path);
-		gtk_selection_data_set(data, data->target, 8, uri, strlen(uri));
-		g_object_set_data_full((GObject *)w, "e-drag-uri", uri, g_free);
+		uri_crlf = g_strconcat(uri, "\r\n", NULL);
+		g_free(uri);
+		gtk_selection_data_set(data, data->target, 8, uri_crlf, strlen(uri_crlf));
+		g_object_set_data_full((GObject *)w, "e-drag-uri", uri_crlf, g_free);
 		break;
 	default:
 		abort();
@@ -1338,7 +1388,12 @@ efhd_drag_data_delete(GtkWidget *w, GdkDragContext *drag, EMFormatHTMLPObject *p
 	if (uri) {
 		/* NB: this doesn't kill the dnd directory */
 		/* NB: is this ever called? */
-		unlink(uri+7);
+		/* NB even more: doesn't the e-drag-uri have \r\n
+		 * appended? (see efhd_drag_data_get())
+		 */
+		char *filename = g_filename_from_uri (uri, NULL, NULL);
+		g_unlink(filename);
+		g_free(filename);
 		g_object_set_data((GObject *)w, "e-drag-uri", NULL);
 	}
 }
@@ -1376,6 +1431,46 @@ efhd_image_resized(GtkWidget *w, GtkAllocation *event, struct _attach_puri *info
 	g_object_unref(pb);
 }
 
+static void
+efhd_change_cursor(GtkWidget *w, GdkEventCrossing *event, struct _attach_puri *info)
+{
+	if (info->shown && info->image) {
+		if (info->fit_width != 0) {
+			if (em_icon_stream_is_resized(info->puri.cid, info->fit_width, info->fit_height))
+			    	e_cursor_set(w->window, E_CURSOR_ZOOM_IN);
+
+		}
+	}
+}
+
+static void
+efhd_image_fit_width(GtkWidget *widget, GdkEventButton *event, struct _attach_puri *info)
+{	
+	int width;
+	
+	width = ((GtkWidget *)((EMFormatHTML *)info->puri.format)->html)->allocation.width - 12;
+
+	if (info->shown && info->image) {
+		if (info->fit_width != 0) {
+			if (em_icon_stream_is_resized(info->puri.cid, info->fit_width, info->fit_height)) {
+				if(info->fit_width != width) {
+					info->fit_width = width;
+					e_cursor_set (widget->window, E_CURSOR_ZOOM_IN);
+				} else {
+					info->fit_width = 0;
+					e_cursor_set(widget->window, E_CURSOR_ZOOM_OUT);
+				}			
+			}
+		} else {
+			info->fit_width = width;
+			e_cursor_set (widget->window, E_CURSOR_ZOOM_IN);			
+		}
+	}
+
+	gtk_image_set_from_pixbuf(info->image, em_icon_stream_get_image(info->puri.cid, info->fit_width, info->fit_height));
+}
+
+
 static gboolean
 efhd_attachment_image(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *pobject)
 {
@@ -1420,7 +1515,9 @@ efhd_attachment_image(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObjec
 	g_signal_connect (box, "drag-data-delete", G_CALLBACK(efhd_drag_data_delete), pobject);
 
 	g_signal_connect(box, "button_press_event", G_CALLBACK(efhd_image_popup), info);
+	g_signal_connect(box, "enter-notify-event", G_CALLBACK(efhd_change_cursor), info);
 	g_signal_connect(box, "popup_menu", G_CALLBACK(efhd_attachment_popup_menu), info);
+	g_signal_connect(box, "button-press-event", G_CALLBACK(efhd_image_fit_width), info);	
 
 	return TRUE;
 }
@@ -1884,13 +1981,14 @@ efhd_attachment_bar_refresh (EMFormatHTMLDisplay *efhd)
 		char *txt;
 
 		/* Cant i put in the number of attachments here ?*/
-		txt = g_strdup_printf(ngettext("%d attachment", "%d attachments", nattachments), nattachments);
-		gtk_label_set_text ((GtkLabel *)efhd->priv->label, txt);
+		txt = g_strdup_printf(ngettext("%d attachme_nt", "%d attachme_nts", nattachments), nattachments);
+		gtk_label_set_text_with_mnemonic ((GtkLabel *)efhd->priv->label, txt);
 		g_free (txt);
 	
 		/* Show the bar even when the first attachment is added */
 		if (nattachments == 1) {
 			gtk_widget_show_all (efhd->priv->attachment_area);
+			gtk_label_set_text_with_mnemonic ((GtkLabel *)efhd->priv->save_txt, _("S_ave"));
 			
 			if (efhd->priv->show_bar) {
 				gtk_widget_show(efhd->priv->down);
@@ -1900,6 +1998,8 @@ efhd_attachment_bar_refresh (EMFormatHTMLDisplay *efhd)
 				gtk_widget_hide(efhd->priv->down);
 				gtk_widget_hide(efhd->priv->attachment_box);
 			}
+		} else if (nattachments > 1) {
+			gtk_label_set_text_with_mnemonic ((GtkLabel *)efhd->priv->save_txt, _("S_ave All"));			
 		}
 	}
 }
@@ -1930,15 +2030,25 @@ efhd_bar_scroll_event(GtkWidget *w, GdkEventScroll *event, EMFormatHTMLDisplay *
 	return TRUE;
 }
 
+gboolean 
+efhd_mnemonic_show_bar (GtkWidget *widget, gboolean focus, GtkWidget *efhd)
+{
+	attachment_bar_arrow_clicked (NULL, efhd);
+
+	return TRUE;
+}
+
 static gboolean
 efhd_add_bar(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *pobject)
 {
 	EMFormatHTMLDisplay *efhd = (EMFormatHTMLDisplay *)efh;
 	struct _EMFormatHTMLDisplayPrivate *priv = efhd->priv;
-	GtkWidget *hbox1, *hbox2, *hbox3, *vbox, *txt, *image, *save;
+	GtkWidget *hbox1, *hbox2, *hbox3, *vbox, *txt, *image, *save, *scroll;
 	int width, height;
 
 	priv->attachment_bar = e_attachment_bar_new(NULL);
+	scroll = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (scroll, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	((EAttachmentBar *)priv->attachment_bar)->expand = TRUE;
 	
 	priv->forward = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
@@ -1947,11 +2057,15 @@ efhd_add_bar(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *pobjec
 	gtk_box_pack_start ((GtkBox *)hbox3, priv->forward, FALSE, FALSE, 0);
 	gtk_box_pack_start ((GtkBox *)hbox3, priv->down, FALSE, FALSE, 0);
 	priv->arrow = (GtkWidget *)gtk_tool_button_new(hbox3, NULL);
+	g_signal_connect (priv->arrow, "mnemonic_activate", G_CALLBACK (efhd_mnemonic_show_bar), efh);
+	atk_object_set_name (gtk_widget_get_accessible (priv->arrow), _("Toggle Attachment Bar"));
 
 	priv->label = gtk_label_new(_("No Attachment"));
+	gtk_label_set_mnemonic_widget (priv->label, priv->arrow);
 	save = gtk_button_new();
 	image = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_BUTTON);
-	txt = gtk_label_new(_("Save All"));
+	txt = gtk_label_new_with_mnemonic(_("S_ave"));
+	priv->save_txt = txt;
 	hbox1 = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start((GtkBox *)hbox1, image, FALSE, FALSE, 2);
 	gtk_box_pack_start((GtkBox *)hbox1, txt, FALSE, FALSE, 0);
@@ -1963,7 +2077,8 @@ efhd_add_bar(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *pobjec
 	gtk_box_pack_start ((GtkBox *)hbox2, priv->label, FALSE, FALSE, 2);
 	gtk_box_pack_start ((GtkBox *)hbox2, save, FALSE, FALSE, 2);
 
-	priv->attachment_box = gtk_frame_new (NULL);
+	priv->attachment_box = scroll;
+	gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 	gtk_container_add ((GtkContainer *)priv->attachment_box, priv->attachment_bar);
 
 	gtk_widget_get_size_request(priv->attachment_bar, &width, &height);
