@@ -36,12 +36,16 @@
 #include <errno.h>
 #include <ctype.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <gtk/gtk.h>
 
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-config.h>
 
 #include <camel/camel.h>
+#include <camel/camel-store.h>
 #include <camel/camel-session.h>
 #include <camel/camel-file-utils.h>
 #include <camel/camel-disco-folder.h>
@@ -52,15 +56,16 @@
 
 #include <libgnome/gnome-i18n.h>
 
-#include <gal/util/e-util.h>
-#include <gal/util/e-iconv.h>
-#include <gal/util/e-xml-utils.h>
+#include <e-util/e-util.h>
+#include <libedataserver/e-data-server-util.h>
+#include <libedataserver/e-iconv.h>
+#include <e-util/e-xml-utils.h>
 
 #include "e-util/e-bconf-map.h"
-#include "e-util/e-account-list.h"
+#include "libedataserver/e-account-list.h"
 #include "e-util/e-signature-list.h"
-#include "e-util/e-path.h"
-#include "widgets/misc/e-error.h"
+#include "e-util/e-error.h"
+#include "e-util/e-util-private.h"
 
 #include "mail-config.h"
 #include "em-utils.h"
@@ -68,7 +73,43 @@
 
 #define d(x) x
 
+#ifndef G_OS_WIN32
+
+/* No previous versions have been available on Win32, so don't bother
+ * with upgrade support from 1.x on Win32.
+ */
+
 /* upgrade helper functions */
+static xmlDocPtr
+emm_load_xml (const char *dirname, const char *filename)
+{
+	xmlDocPtr doc;
+	struct stat st;
+	char *path;
+	
+	path = g_strdup_printf ("%s/%s", dirname, filename);
+	if (stat (path, &st) == -1 || !(doc = xmlParseFile (path))) {
+		g_free (path);
+		return NULL;
+	}
+	
+	g_free (path);
+	
+	return doc;
+}
+
+static int
+emm_save_xml (xmlDocPtr doc, const char *dirname, const char *filename)
+{
+	char *path;
+	int retval;
+	
+	path = g_strdup_printf ("%s/%s", dirname, filename);
+	retval = e_xml_save_file (path, doc);
+	g_free (path);
+	
+	return retval;
+}
 
 static xmlNodePtr
 xml_find_node (xmlNodePtr parent, const char *name)
@@ -1374,6 +1415,8 @@ nofile:
 	return g_strdup_printf("%s:%s", protocol, dirname);
 }
 
+#endif	/* !G_OS_WIN32 */
+
 enum {
 	CP_UNIQUE = 0,
 	CP_OVERWRITE,
@@ -1398,16 +1441,16 @@ cp (const char *src, const char *dest, gboolean show_progress, int mode)
 
 	/* if the dest file exists and has content, abort - we don't
 	 * want to corrupt their existing data */
-	if (stat (dest, &st) == 0 && st.st_size > 0 && mode == CP_UNIQUE) {
+	if (g_stat (dest, &st) == 0 && st.st_size > 0 && mode == CP_UNIQUE) {
 		errno = EEXIST;
 		return -1;
 	}
 	
-	if (stat (src, &st) == -1
-	    || (readfd = open (src, O_RDONLY)) == -1)
+	if (g_stat (src, &st) == -1
+	    || (readfd = g_open (src, O_RDONLY | O_BINARY, 0)) == -1)
 		return -1;
 
-	if ((writefd = open (dest, open_flags[mode], 0666)) == -1) {
+	if ((writefd = g_open (dest, open_flags[mode] | O_BINARY, 0666)) == -1) {
 		errnosav = errno;
 		close (readfd);
 		errno = errnosav;
@@ -1432,9 +1475,10 @@ cp (const char *src, const char *dest, gboolean show_progress, int mode)
 			goto exception;
 		
 		total += nwritten;
-		
+#ifndef G_OS_WIN32		
 		if (show_progress)
 			em_migrate_set_progress (((double) total) / ((double) st.st_size));
+#endif
 	} while (total < st.st_size);
 	
 	if (fsync (writefd) == -1)
@@ -1467,6 +1511,8 @@ cp (const char *src, const char *dest, gboolean show_progress, int mode)
 	return -1;
 }
 
+#ifndef G_OS_WIN32
+
 static int
 cp_r (const char *src, const char *dest, const char *pattern, int mode)
 {
@@ -1476,7 +1522,7 @@ cp_r (const char *src, const char *dest, const char *pattern, int mode)
 	struct stat st;
 	DIR *dir;
 	
-	if (camel_mkdir (dest, 0777) == -1)
+	if (e_util_mkdir_hier (dest, 0777) == -1)
 		return -1;
 	
 	if (!(dir = opendir (src)))
@@ -1597,7 +1643,7 @@ em_migrate_folder(EMMigrateSession *session, const char *dirname, const char *fu
 		slen = src->len;
 		dlen = dest->len;
 		
-		if (camel_mkdir (dest->str, 0777) == -1 && errno != EEXIST) {
+		if (e_util_mkdir_hier (dest->str, 0777) == -1 && errno != EEXIST) {
 			camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Unable to create new folder `%s': %s"),
 					     dest->str, g_strerror(errno));
@@ -2026,7 +2072,7 @@ em_migrate_pop_uid_caches_1_4 (const char *evolution_dir, CamelException *ex)
 	g_free (cache_dir);
 	
 	cache_dir = g_build_filename (evolution_dir, "mail", "pop", NULL);
-	if (camel_mkdir (cache_dir, 0777) == -1) {
+	if (e_util_mkdir_hier (cache_dir, 0777) == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Unable to create POP3 keep-on-server data directory `%s': %s"),
 				      cache_dir, g_strerror(errno));
@@ -2054,7 +2100,7 @@ em_migrate_pop_uid_caches_1_4 (const char *evolution_dir, CamelException *ex)
 		/* strip the trailing '_' */
 		g_string_truncate (newpath, newpath->len - 1);
 		
-		if (camel_mkdir (newpath->str, 0777) == -1
+		if (e_util_mkdir_hier (newpath->str, 0777) == -1
 		    || cp(oldpath->str, (g_string_append(newpath, "/uid-cache"))->str, FALSE, CP_UNIQUE)) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("Unable to copy POP3 keep-on-server data `%s': %s"),
@@ -2114,7 +2160,7 @@ em_migrate_folder_expand_state_1_4 (const char *evolution_dir, CamelException *e
 	
 	destpath = g_string_new (evolution_dir);
 	g_string_append (destpath, "/mail/config");
-	if (camel_mkdir (destpath->str, 0777) == -1 || !(dir = opendir (srcpath->str))) {
+	if (e_util_mkdir_hier (destpath->str, 0777) == -1 || !(dir = opendir (srcpath->str))) {
 		g_string_free (destpath, TRUE);
 		g_string_free (srcpath, TRUE);
 		return 0;
@@ -2212,7 +2258,7 @@ em_migrate_folder_view_settings_1_4 (const char *evolution_dir, CamelException *
 	
 	destpath = g_string_new (evolution_dir);
 	g_string_append (destpath, "/mail/views");
-	if (camel_mkdir (destpath->str, 0777) == -1 || !(dir = opendir (srcpath->str))) {
+	if (e_util_mkdir_hier (destpath->str, 0777) == -1 || !(dir = opendir (srcpath->str))) {
 		g_string_free (destpath, TRUE);
 		g_string_free (srcpath, TRUE);
 		return 0;
@@ -2315,6 +2361,84 @@ em_migrate_folder_view_settings_1_4 (const char *evolution_dir, CamelException *
 	return 0;
 }
 
+#define SUBFOLDER_DIR_NAME     "subfolders"
+#define SUBFOLDER_DIR_NAME_LEN 10
+
+static char *
+e_path_to_physical (const char *prefix, const char *vpath)
+{
+	const char *p, *newp;
+	char *dp;
+	char *ppath;
+	int ppath_len;
+	int prefix_len;
+
+	while (*vpath == '/')
+		vpath++;
+	if (!prefix)
+		prefix = "";
+
+	/* Calculate the length of the real path. */
+	ppath_len = strlen (vpath);
+	ppath_len++;	/* For the ending zero.  */
+
+	prefix_len = strlen (prefix);
+	ppath_len += prefix_len;
+	ppath_len++;	/* For the separating slash.  */
+
+	/* Take account of the fact that we need to translate every
+	 * separator into `subfolders/'.
+	 */
+	p = vpath;
+	while (1) {
+		newp = strchr (p, '/');
+		if (newp == NULL)
+			break;
+
+		ppath_len += SUBFOLDER_DIR_NAME_LEN;
+		ppath_len++; /* For the separating slash.  */
+
+		/* Skip consecutive slashes.  */
+		while (*newp == '/')
+			newp++;
+
+		p = newp;
+	};
+
+	ppath = g_malloc (ppath_len);
+	dp = ppath;
+
+	memcpy (dp, prefix, prefix_len);
+	dp += prefix_len;
+	*(dp++) = '/';
+
+	/* Copy the mangled path.  */
+	p = vpath;
+ 	while (1) {
+		newp = strchr (p, '/');
+		if (newp == NULL) {
+			strcpy (dp, p);
+			break;
+		}
+
+		memcpy (dp, p, newp - p + 1); /* `+ 1' to copy the slash too.  */
+		dp += newp - p + 1;
+
+		memcpy (dp, SUBFOLDER_DIR_NAME, SUBFOLDER_DIR_NAME_LEN);
+		dp += SUBFOLDER_DIR_NAME_LEN;
+
+		*(dp++) = '/';
+
+		/* Skip consecutive slashes.  */
+		while (*newp == '/')
+			newp++;
+
+		p = newp;
+	}
+
+	return ppath;
+}
+
 static int
 em_migrate_imap_cmeta_1_4(const char *evolution_dir, CamelException *ex)
 {
@@ -2350,7 +2474,7 @@ em_migrate_imap_cmeta_1_4(const char *evolution_dir, CamelException *ex)
 							       url->host?url->host:"");
 
 					dir = e_path_to_physical(base, path);
-					if (camel_mkdir(dir, 0777) == 0) {
+					if (e_util_mkdir_hier(dir, 0777) == 0) {
 						char *cmeta;
 						FILE *fp;
 
@@ -2392,6 +2516,42 @@ em_migrate_imap_cmeta_1_4(const char *evolution_dir, CamelException *ex)
 	return 0;
 }
 
+static void
+remove_system_searches(xmlDocPtr searches)
+{
+	xmlNodePtr node;
+
+	/* in pre 2.0, system searches were stored in the user
+	 * searches.xml file with the source set to 'demand'.  In 2.0+
+	 * the system searches are stored in the system
+	 * searchtypes.xml file instead */
+
+	node = xmlDocGetRootElement(searches);
+	if (!node->name || strcmp(node->name, "filteroptions"))
+		return;
+
+	if (!(node = xml_find_node(node, "ruleset")))
+		return;
+	
+	node = node->children;
+	while (node != NULL) {
+		xmlNodePtr nnode = node->next;
+
+		if (node->name && !strcmp (node->name, "rule")) {
+			char *src;
+
+			src = xmlGetProp(node, "source");
+			if (src && !strcmp(src, "demand")) {
+				xmlUnlinkNode(node);
+				xmlFreeNodeList(node);
+			}
+			xmlFree (src);
+		}
+		
+		node = nnode;
+	}	
+}
+
 static int
 em_migrate_1_4 (const char *evolution_dir, xmlDocPtr filters, xmlDocPtr vfolders, CamelException *ex)
 {
@@ -2399,10 +2559,12 @@ em_migrate_1_4 (const char *evolution_dir, xmlDocPtr filters, xmlDocPtr vfolders
 	CamelException lex;
 	struct stat st;
 	char *path;
-	
+	xmlDocPtr searches;
+
 	path = g_build_filename (evolution_dir, "mail", NULL);
 
 	camel_init (path, TRUE);
+	camel_provider_init();
 	session = (EMMigrateSession *) em_migrate_session_new (path);
 	g_free (path);
 	
@@ -2410,7 +2572,7 @@ em_migrate_1_4 (const char *evolution_dir, xmlDocPtr filters, xmlDocPtr vfolders
 	
 	path = g_strdup_printf ("mbox:%s/.evolution/mail/local", g_get_home_dir ());
 	if (stat (path + 5, &st) == -1) {
-		if (errno != ENOENT || camel_mkdir (path + 5, 0777) == -1) {
+		if (errno != ENOENT || e_util_mkdir_hier (path + 5, 0777) == -1) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("Failed to create local mail storage `%s': %s"),
 					      path + 5, g_strerror (errno));
@@ -2447,16 +2609,17 @@ em_migrate_1_4 (const char *evolution_dir, xmlDocPtr filters, xmlDocPtr vfolders
 	upgrade_xml_uris(filters, upgrade_xml_uris_1_4);
 	upgrade_vfolder_sources_1_4(vfolders);
 	upgrade_xml_uris(vfolders, upgrade_xml_uris_1_4);
-	
-	path = g_build_filename (g_get_home_dir (), "evolution", "searches.xml", NULL);
-	if (stat (path, &st) == 0 && S_ISREG (st.st_mode)) {
-		char *dest;
-		
-		dest = g_build_filename (evolution_dir, "mail", "searches.xml", NULL);
-		cp (path, dest, FALSE, CP_UNIQUE);
-		g_free (dest);
+
+	path = g_build_filename(g_get_home_dir(), "evolution", NULL);
+	searches = emm_load_xml(path, "searches.xml");
+	g_free(path);
+	if (searches) {
+		remove_system_searches(searches);
+		path = g_build_filename(evolution_dir, "mail", NULL);
+		emm_save_xml(searches, path, "searches.xml");
+		g_free(path);
+		xmlFreeDoc(searches);
 	}
-	g_free (path);
 	
 	if (em_migrate_pop_uid_caches_1_4 (evolution_dir, ex) == -1)
 		return -1;
@@ -2474,43 +2637,13 @@ em_migrate_1_4 (const char *evolution_dir, xmlDocPtr filters, xmlDocPtr vfolders
 	return 0;
 }
 
-
-static xmlDocPtr
-emm_load_xml (const char *dirname, const char *filename)
-{
-	xmlDocPtr doc;
-	struct stat st;
-	char *path;
-	
-	path = g_strdup_printf ("%s/%s", dirname, filename);
-	if (stat (path, &st) == -1 || !(doc = xmlParseFile (path))) {
-		g_free (path);
-		return NULL;
-	}
-	
-	g_free (path);
-	
-	return doc;
-}
-
-static int
-emm_save_xml (xmlDocPtr doc, const char *dirname, const char *filename)
-{
-	char *path;
-	int retval;
-	
-	path = g_strdup_printf ("%s/%s", dirname, filename);
-	retval = e_xml_save_file (path, doc);
-	g_free (path);
-	
-	return retval;
-}
+#endif
 
 static int
 emm_setup_initial(const char *evolution_dir)
 {
-	DIR *dir;
-	struct dirent *d;
+	GDir *dir;
+	const char *d;
 	struct stat st;
 	const GList *l;
 	char *local, *base;
@@ -2520,8 +2653,8 @@ emm_setup_initial(const char *evolution_dir)
 
 	d(printf("Setting up initial mail tree\n"));
 	
-	base = g_build_filename(evolution_dir, "/mail/local", NULL);
-	if (camel_mkdir(base, 0777) == -1 && errno != EEXIST) {
+	base = g_build_filename(evolution_dir, "mail/local", NULL);
+	if (e_util_mkdir_hier(base, 0777) == -1 && errno != EEXIST) {
 		g_free(base);
 		return -1;
 	}
@@ -2530,32 +2663,33 @@ emm_setup_initial(const char *evolution_dir)
 	for (l = gnome_i18n_get_language_list("LC_MESSAGES");
 	     l != NULL;
 	     l = g_list_next(l)) {
-		local = g_build_filename(EVOLUTION_PRIVDATADIR "/default", (char *)l->data, "mail/local", NULL);
-		if (stat(local, &st) == 0)
+		local = g_build_filename(EVOLUTION_PRIVDATADIR,
+					 "default",
+					 (char *)l->data,
+					 "mail/local", NULL);
+		if (g_stat(local, &st) == 0)
 			goto gotlocal;
 
 		g_free(local);
 	}
 
-	local = g_build_filename(EVOLUTION_PRIVDATADIR "/default/C/mail/local", NULL);
+	local = g_build_filename(EVOLUTION_PRIVDATADIR,
+				 "default/C/mail/local", NULL);
 gotlocal:
 
-	dir = opendir(local);
+	dir = g_dir_open(local, 0, NULL);
 	if (dir) {
-		while ((d = readdir(dir))) {
+		while ((d = g_dir_read_name(dir))) {
 			char *src, *dest;
 
-			if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
-				continue;
-
-			src = g_build_filename(local, d->d_name, NULL);
-			dest = g_build_filename(base, d->d_name, NULL);
+			src = g_build_filename(local, d, NULL);
+			dest = g_build_filename(base, d, NULL);
 
 			cp(src, dest, FALSE, CP_UNIQUE);
 			g_free(dest);
 			g_free(src);
 		}
-		closedir(dir);
+		g_dir_close(dir);
 	}
 
 	g_free(local);
@@ -2572,8 +2706,8 @@ em_migrate (const char *evolution_dir, int major, int minor, int revision, Camel
 	
 	/* make sure ~/.evolution/mail exists */
 	path = g_build_filename (evolution_dir, "mail", NULL);
-	if (stat (path, &st) == -1) {
-		if (errno != ENOENT || camel_mkdir (path, 0777) == -1) {
+	if (g_stat (path, &st) == -1) {
+		if (errno != ENOENT || e_util_mkdir_hier (path, 0777) == -1) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, 
 					      _("Unable to create local mail folders at `%s': %s"),
 					      path, g_strerror (errno));
@@ -2586,7 +2720,8 @@ em_migrate (const char *evolution_dir, int major, int minor, int revision, Camel
 	
 	if (major == 0)
 		return emm_setup_initial(evolution_dir);
-	
+
+#ifndef G_OS_WIN32
 	if (major == 1 && minor < 5) {
 		xmlDocPtr config_xmldb = NULL, filters, vfolders;
 		
@@ -2644,6 +2779,6 @@ em_migrate (const char *evolution_dir, int major, int minor, int revision, Camel
 		
 		g_free (path);
 	}
-	
+#endif	/* !G_OS_WIN32 */
 	return 0;
 }
