@@ -35,6 +35,7 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include <libedataserver/e-data-server-util.h>
 #include "em-utils.h"
 #include "em-composer-utils.h"
 #include "em-format.h"
@@ -48,6 +49,7 @@
 
 #include "misc/e-info-label.h"
 #include "e-util/e-error.h"
+#include "e-util/e-util-private.h"
 
 #include "em-search-context.h"
 #include "mail-config.h"
@@ -63,6 +65,7 @@
 
 #include "e-activity-handler.h"
 #include "shell/e-user-creatable-items-handler.h"
+#include "shell/e-component-view.h"
 
 #include "composer/e-msg-composer.h"
 
@@ -71,6 +74,7 @@
 #include <gtk/gtklabel.h>
 
 #include <e-util/e-mktemp.h>
+#include <Evolution.h>
 
 #include <table/e-tree.h>
 #include <table/e-tree-memory.h>
@@ -93,6 +97,9 @@ static void create_local_item_cb(EUserCreatableItemsHandler *handler, const char
 #define PARENT_TYPE evolution_component_get_type ()
 static BonoboObjectClass *parent_class = NULL;
 
+#define OFFLINE 0
+#define ONLINE 1
+
 struct _store_info {	
 	CamelStore *store;
 	char *name;
@@ -342,7 +349,7 @@ setup_search_context (MailComponent *component)
 
 	if (priv->search_context == NULL) {
 		char *user = g_build_filename(component->priv->base_directory, "mail/searches.xml", NULL);
-		char *system = g_strdup (EVOLUTION_PRIVDATADIR "/searchtypes.xml");
+		char *system = g_build_filename (EVOLUTION_PRIVDATADIR, "searchtypes.xml", NULL);
 		
 		priv->search_context = (RuleContext *)em_search_context_new ();
 		g_object_set_data_full (G_OBJECT (priv->search_context), "user", user, g_free);
@@ -490,10 +497,15 @@ view_on_url (GObject *emitter, const char *url, const char *nice_url, MailCompon
 }
 
 static void
-view_changed(EMFolderView *emfv, EInfoLabel *el)
+view_changed(EMFolderView *emfv, EComponentView *component_view)
 {
+	EInfoLabel *el = g_object_get_data((GObject *)component_view, "info-label");
+	CORBA_Environment ev;
+
+	CORBA_exception_init(&ev);
+
 	if (emfv->folder) {
-		char *name;
+		char *name, *title;
 		guint32 visible, unread, deleted, junked;
 		GString *tmp = g_string_new("");
 
@@ -555,20 +567,27 @@ view_changed(EMFolderView *emfv, EInfoLabel *el)
 			e_info_label_set_info(el, _(name), tmp->str);
 		else
 			e_info_label_set_info(el, name, tmp->str);
+
+		title = g_strdup_printf("%s (%s)", name, tmp->str);
+		e_component_view_set_title(component_view, title);
+		g_free(title);
+
 		g_string_free(tmp, TRUE);
 		camel_object_free(emfv->folder, CAMEL_FOLDER_NAME, name);
 	} else {
 		e_info_label_set_info(el, _("Mail"), "");
+		e_component_view_set_title(component_view, _("Mail"));
 	}
 }
 
 static int
 view_changed_timeout(void *d)
 {
-	EInfoLabel *el = d;
+	EComponentView *component_view = d;	
+	EInfoLabel *el = g_object_get_data((GObject *)component_view, "info-label");
 	EMFolderView *emfv = g_object_get_data((GObject *)el, "folderview");
 
-	view_changed(emfv, el);
+	view_changed(emfv, component_view);
 
 	g_object_set_data((GObject *)emfv, "view-changed-timeout", NULL);
 
@@ -579,9 +598,10 @@ view_changed_timeout(void *d)
 }
 
 static void
-view_changed_cb(EMFolderView *emfv, EInfoLabel *el)
+view_changed_cb(EMFolderView *emfv, EComponentView *component_view)
 {
 	void *v;
+	EInfoLabel *el = g_object_get_data((GObject *)component_view, "info-label");
 
 	/* This can get called 3 times every cursor move, so
 	   we don't need to/want to run it immediately */
@@ -595,23 +615,19 @@ view_changed_cb(EMFolderView *emfv, EInfoLabel *el)
 		g_object_ref(el);
 	}
 
-	g_object_set_data((GObject *)emfv, "view-changed-timeout", GINT_TO_POINTER(g_timeout_add(250, view_changed_timeout, el)));
+	g_object_set_data((GObject *)emfv, "view-changed-timeout", GINT_TO_POINTER(g_timeout_add(250, view_changed_timeout, component_view)));
 }
 
 /* Evolution::Component CORBA methods.  */
 
-static void
-impl_createControls (PortableServer_Servant servant,
-		     Bonobo_Control *corba_tree_control,
-		     Bonobo_Control *corba_view_control,
-		     Bonobo_Control *corba_statusbar_control,
-		     CORBA_Environment *ev)
+static GNOME_Evolution_ComponentView
+impl_createView (PortableServer_Servant servant,
+		 GNOME_Evolution_ShellView parent,
+		 CORBA_Environment *ev)
 {
 	MailComponent *mail_component = MAIL_COMPONENT (bonobo_object_from_servant (servant));
 	MailComponentPrivate *priv = mail_component->priv;
-	BonoboControl *tree_control;
-	BonoboControl *view_control;
-	BonoboControl *statusbar_control;
+	EComponentView *component_view;
 	GtkWidget *tree_widget, *vbox, *info;
 	GtkWidget *view_widget;
 	GtkWidget *statusbar_widget;
@@ -652,25 +668,23 @@ impl_createControls (PortableServer_Servant servant,
 	gtk_widget_show(info);
 	gtk_widget_show(vbox);
 
-	tree_control = bonobo_control_new (vbox);
-	view_control = bonobo_control_new (view_widget);
-	statusbar_control = bonobo_control_new (statusbar_widget);
-	
-	*corba_tree_control = CORBA_Object_duplicate (BONOBO_OBJREF (tree_control), ev);
-	*corba_view_control = CORBA_Object_duplicate (BONOBO_OBJREF (view_control), ev);
-	*corba_statusbar_control = CORBA_Object_duplicate (BONOBO_OBJREF (statusbar_control), ev);
+	component_view = e_component_view_new(parent, "mail", vbox, view_widget, statusbar_widget);
+
+	g_object_set_data((GObject *)component_view, "info-label", info);
 
 	g_object_set_data_full((GObject *)view_widget, "e-creatable-items-handler",
 			       e_user_creatable_items_handler_new("mail", create_local_item_cb, tree_widget),
 			       (GDestroyNotify)g_object_unref);
 
-	g_signal_connect (view_control, "activate", G_CALLBACK (view_control_activate_cb), view_widget);
+	g_signal_connect (component_view->view_control, "activate", G_CALLBACK (view_control_activate_cb), view_widget);
 	g_signal_connect (tree_widget, "folder-selected", G_CALLBACK (folder_selected_cb), view_widget);
 
-	g_signal_connect(view_widget, "changed", G_CALLBACK(view_changed_cb), info);
-	g_signal_connect(view_widget, "loaded", G_CALLBACK(view_changed_cb), info);
-
+	g_signal_connect(view_widget, "changed", G_CALLBACK(view_changed_cb), component_view);
+	g_signal_connect(view_widget, "loaded", G_CALLBACK(view_changed_cb), component_view);
+	
 	g_object_set_data((GObject*)info, "folderview", view_widget);
+
+	return BONOBO_OBJREF(component_view);
 }
 
 static CORBA_boolean
@@ -773,7 +787,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant, CORBA_Environment
 	list->_buffer[1].description = _("New Mail Folder");
 	list->_buffer[1].menuDescription = _("Mail _Folder");
 	list->_buffer[1].tooltip = _("Create a new mail folder");
-	list->_buffer[1].menuShortcut = 'f';
+	list->_buffer[1].menuShortcut = '\0';
 	list->_buffer[1].iconName = "stock_new-dir";
 	list->_buffer[1].type = GNOME_Evolution_CREATABLE_FOLDER;
 
@@ -964,10 +978,36 @@ setline_check(void *key, void *value, void *data)
 	}
 }
 
+int 
+status_check (GNOME_Evolution_ShellState shell_state)
+{
+	int status;
+
+	switch (shell_state)
+	{
+	    case GNOME_Evolution_USER_OFFLINE:
+		    status = OFFLINE;
+		    break;
+	    case GNOME_Evolution_FORCED_OFFLINE: 
+		    /*Network is down so change network state on the camel session*/
+		    status = OFFLINE;
+		    /* Cancel all operations as they wont happen anyway cos Network is down*/
+		    mail_cancel_all ();
+		    camel_session_set_network_state (session, FALSE);
+		    break;
+	    case GNOME_Evolution_USER_ONLINE:
+		    camel_session_set_network_state (session, TRUE);
+		    status = ONLINE;
+	}	  
+
+	return status;
+}
+
 static void
-impl_setLineStatus(PortableServer_Servant servant, CORBA_boolean status, GNOME_Evolution_Listener listener, CORBA_Environment *ev)
+impl_setLineStatus(PortableServer_Servant servant, GNOME_Evolution_ShellState shell_state, GNOME_Evolution_Listener listener, CORBA_Environment *ev)
 {
 	struct _setline_data *sd;
+	int status = status_check(shell_state);
 
 	/* This will dis/enable further auto-mail-check action. */
 	/* FIXME: If send/receive active, wait for it to finish? */
@@ -1016,7 +1056,7 @@ mail_component_class_init (MailComponentClass *class)
 	object_class->dispose  = impl_dispose;
 	object_class->finalize = impl_finalize;
 	
-	epv->createControls          = impl_createControls;
+	epv->createView          = impl_createView;
 	epv->requestQuit = impl_requestQuit;
 	epv->quit = impl_quit;
 	epv->_get_userCreatableItems = impl__get_userCreatableItems;
@@ -1040,7 +1080,14 @@ mail_component_init (MailComponent *component)
 	priv->lock = g_mutex_new();
 
 	priv->base_directory = g_build_filename (g_get_home_dir (), ".evolution", NULL);
-	if (camel_mkdir (priv->base_directory, 0777) == -1 && errno != EEXIST)
+#ifdef G_OS_WIN32
+	{
+		char *p = priv->base_directory;
+		while ((p = strchr(p, '\\')))
+			*p++ = '/';
+	}
+#endif
+	if (e_util_mkdir_hier (priv->base_directory, 0777) == -1 && errno != EEXIST)
 		abort ();
 	
 	priv->model = em_folder_tree_model_new (priv->base_directory);
