@@ -33,14 +33,22 @@
 #include <string.h>
 #include <time.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <libgnome/gnome-sound.h>
+#include <libgnome/gnome-i18n.h>
 #include <bonobo/bonobo-exception.h>
 #include <camel/camel-store.h>
 #include <camel/camel-folder.h>
 #include <camel/camel-vtrash-folder.h>
 #include <camel/camel-vee-store.h>
+#include <camel/camel-offline-store.h>
 #include <camel/camel-disco-store.h>
 
+#include <libedataserver/e-data-server-util.h>
+#include "e-util/e-util.h"
+
 #include "mail-mt.h"
 #include "mail-folder-cache.h"
 #include "mail-ops.h"
@@ -53,8 +61,12 @@
 #include "mail-config.h"
 #include "em-folder-tree-model.h"
 
-#define w(x) 
-#define d(x) x
+#include "em-event.h"
+
+#define w(x)
+#define d(x)
+
+/* This code is a mess, there is no reason it should be so complicated. */
 
 /* note that many things are effectively serialised by having them run in
    the main loop thread which they need to do because of corba/gtk calls */
@@ -66,7 +78,6 @@ static pthread_mutex_t info_lock = PTHREAD_MUTEX_INITIALIZER;
 struct _folder_info {
 	struct _store_info *store_info;	/* 'parent' link */
 
-	char *path;		/* shell path */
 	char *full_name;	/* full name of folder/folderinfo */
 	char *uri;		/* uri of folder */
 	
@@ -86,10 +97,9 @@ struct _folder_update {
 	unsigned int unsub:1;   /* unsubcribing? */
 	unsigned int new:1;     /* new mail arrived? */
 
-	char *path;
-	char *name;
+	char *full_name;
 	char *uri;
-	char *oldpath;
+	char *oldfull;
 	char *olduri;
 
 	int unread;
@@ -135,12 +145,11 @@ static int count_trash = FALSE;
 static void
 free_update(struct _folder_update *up)
 {
-	g_free(up->path);
-	g_free(up->name);
+	g_free(up->full_name);
 	g_free(up->uri);
 	if (up->store)
 		camel_object_unref(up->store);
-	g_free(up->oldpath);
+	g_free(up->oldfull);
 	g_free(up->olduri);
 	g_free(up);
 }
@@ -188,7 +197,6 @@ real_flush_updates(void *o, void *event_data, void *data)
 	struct _MailComponent *component;
 	struct _EMFolderTreeModel *model;
 	struct _folder_update *up;
-	struct _store_info *si;
 	time_t now;
 	
 	component = mail_component_peek ();
@@ -196,8 +204,6 @@ real_flush_updates(void *o, void *event_data, void *data)
 	
 	LOCK(info_lock);
 	while ((up = (struct _folder_update *)e_dlist_remhead(&updates))) {
-		si = g_hash_table_lookup(stores, up->store);
-		
 		UNLOCK(info_lock);
 
 		if (up->remove) {
@@ -223,7 +229,7 @@ real_flush_updates(void *o, void *event_data, void *data)
 		}
 		
 		/* update unread counts */
-		em_folder_tree_model_set_unread_count (model, up->store, up->path, up->unread);
+		em_folder_tree_model_set_unread_count (model, up->store, up->full_name, up->unread);
 		
 		/* new mail notification */
 		if (notify_type == -1) {
@@ -242,6 +248,19 @@ real_flush_updates(void *o, void *event_data, void *data)
 		if (notify_type != 0 && up->new && notify_idle_id == 0 && (now - last_notify >= 5))
 			notify_idle_id = g_idle_add_full (G_PRIORITY_LOW, notify_idle_cb, NULL, NULL);
 		
+		if (up->uri) {
+			EMEvent *e = em_event_peek();
+			EMEventTargetFolder *t = em_event_target_new_folder(e, up->uri, up->new?EM_EVENT_FOLDER_NEWMAIL:0);
+
+			/** @Event: folder.changed
+			 * @Title: Folder changed
+			 * @Target: EMEventTargetFolder
+			 * 
+			 * folder.changed is emitted whenever a folder changes.  There is no detail on how the folder has changed.
+			 */
+			e_event_emit((EEvent *)e, "folder.changed", (EEventTarget *)t);
+		}
+
 		free_update(up);
 		
 		LOCK(info_lock);
@@ -267,9 +286,9 @@ unset_folder_info(struct _folder_info *mfi, int delete, int unsub)
 	if (mfi->folder) {
 		CamelFolder *folder = mfi->folder;
 
-		camel_object_unhook_event(folder, "folder_changed", folder_changed, mfi);
-		camel_object_unhook_event(folder, "renamed", folder_renamed, mfi);
-		camel_object_unhook_event(folder, "finalize", folder_finalised, mfi);
+		camel_object_unhook_event(folder, "folder_changed", folder_changed, NULL);
+		camel_object_unhook_event(folder, "renamed", folder_renamed, NULL);
+		camel_object_unhook_event(folder, "finalize", folder_finalised, NULL);
 	}
 
 	if ((mfi->flags & CAMEL_FOLDER_NOSELECT) == 0) {
@@ -279,7 +298,7 @@ unset_folder_info(struct _folder_info *mfi, int delete, int unsub)
 		up->delete = delete;
 		up->unsub = unsub;
 		up->store = mfi->store_info->store;
-		up->path = g_strdup (mfi->path);
+		up->full_name = g_strdup (mfi->full_name);
 		camel_object_ref(up->store);
 		up->uri = g_strdup(mfi->uri);
 
@@ -291,7 +310,6 @@ unset_folder_info(struct _folder_info *mfi, int delete, int unsub)
 static void
 free_folder_info(struct _folder_info *mfi)
 {
-	g_free(mfi->path);
 	g_free(mfi->full_name);
 	g_free(mfi->uri);
 	g_free(mfi);
@@ -335,10 +353,12 @@ update_1folder(struct _folder_info *mfi, int new, CamelFolderInfo *info)
 		d(printf("update 1 folder '%s'\n", folder->full_name));
 		if ((count_trash && (CAMEL_IS_VTRASH_FOLDER (folder)))
 		    || folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX)
+		    || folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_DRAFTS)
 		    || (count_sent && folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_SENT))) {
 			d(printf(" total count\n"));
 			unread = camel_folder_get_message_count (folder);
-			if (folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX)) {
+			if (folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX) 
+					|| folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_DRAFTS)) {
 				if ((deleted = camel_folder_get_deleted_message_count (folder)) > 0)
 					unread -= deleted;
 			}
@@ -358,10 +378,11 @@ update_1folder(struct _folder_info *mfi, int new, CamelFolderInfo *info)
 		return;
 
 	up = g_malloc0(sizeof(*up));
-	up->path = g_strdup(mfi->path);
+	up->full_name = g_strdup(mfi->full_name);
 	up->unread = unread;
 	up->new = new ? 1 : 0;
 	up->store = mfi->store_info->store;
+	up->uri = g_strdup(mfi->uri);
 	camel_object_ref(up->store);
 	e_dlist_addtail(&updates, (EDListNode *)up);
 	flush_updates();
@@ -377,10 +398,8 @@ setup_folder(CamelFolderInfo *fi, struct _store_info *si)
 	if (mfi) {
 		update_1folder(mfi, 0, fi);
 	} else {
-		/* always 'add it', but only 'add it' to non-local stores */
-		/*d(printf("Adding new folder: %s (%s) %d unread\n", fi->path, fi->url, fi->unread_message_count));*/
+		d(printf("Adding new folder: %s (%s)\n", fi->full_name, fi->uri));
 		mfi = g_malloc0(sizeof(*mfi));
-		mfi->path = g_strdup(fi->path);
 		mfi->full_name = g_strdup(fi->full_name);
 		mfi->uri = g_strdup(fi->uri);
 		mfi->store_info = si;
@@ -390,10 +409,9 @@ setup_folder(CamelFolderInfo *fi, struct _store_info *si)
 		g_hash_table_insert(si->folders_uri, mfi->uri, mfi);
 
 		up = g_malloc0(sizeof(*up));
-		up->path = g_strdup(mfi->path);
-		up->name = g_strdup(fi->name);
+		up->full_name = g_strdup(mfi->full_name);
 		up->uri = g_strdup(fi->uri);
-		up->unread = (fi->unread==-1)?0:fi->unread;
+		up->unread = fi->unread;
 		up->store = si->store;
 		camel_object_ref(up->store);
 		
@@ -424,47 +442,58 @@ static void
 folder_changed (CamelObject *o, gpointer event_data, gpointer user_data)
 {
 	CamelFolderChangeInfo *changes = event_data;
-	CamelFolder *folder = (CamelFolder *) o;
-	struct _folder_info *mfi = user_data;
+	CamelFolder *folder = (CamelFolder *)o;
+	CamelStore *store = folder->parent_store;
+	struct _store_info *si;
+	struct _folder_info *mfi;
 	int new = 0;
 	
-	if (mfi->folder != folder)
-		return;
-
 	d(printf("folder '%s' changed\n", folder->full_name));
 
-	if (!CAMEL_IS_VTRASH_FOLDER(folder)
+	if (!CAMEL_IS_VEE_FOLDER(folder)
 	    && folder != mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX)
+	    && folder != mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_DRAFTS)
 	    && folder != mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_SENT)
 	    && changes && changes->uid_added)
 		new = changes->uid_added->len;
 	
 	LOCK(info_lock);
-	update_1folder(mfi, new, NULL);
+	if (stores != NULL
+	    && (si = g_hash_table_lookup(stores, store)) != NULL
+	    && (mfi = g_hash_table_lookup(si->folders, folder->full_name)) != NULL
+	    && mfi->folder == folder) {
+		update_1folder(mfi, new, NULL);
+	}
 	UNLOCK(info_lock);
 }
 
 static void
 folder_finalised(CamelObject *o, gpointer event_data, gpointer user_data)
 {
-	struct _folder_info *mfi = user_data;
+	CamelFolder *folder = (CamelFolder *)o;
+	CamelStore *store = folder->parent_store;
+	struct _store_info *si;
+	struct _folder_info *mfi;
 
 	d(printf("Folder finalised '%s'!\n", ((CamelFolder *)o)->full_name));
 	LOCK(info_lock);
-	mfi->folder = NULL;
+	if (stores != NULL
+	    && (si = g_hash_table_lookup(stores, store)) != NULL
+	    && (mfi = g_hash_table_lookup(si->folders, folder->full_name)) != NULL
+	    && mfi->folder == folder) {
+		mfi->folder = NULL;
+	}
 	UNLOCK(info_lock);
 }
 
 static void
 folder_renamed(CamelObject *o, gpointer event_data, gpointer user_data)
 {
-	struct _folder_info *mfi = user_data;
 	CamelFolder *folder = (CamelFolder *)o;
 	char *old = event_data;
 
 	d(printf("Folder renamed from '%s' to '%s'\n", old, folder->full_name));
 
-	mfi = mfi;
 	old = old;
 	folder = folder;
 	/* Dont do anything, do it from the store rename event? */
@@ -495,13 +524,13 @@ void mail_note_folder(CamelFolder *folder)
 
 	mfi->folder = folder;
 
-	camel_object_hook_event(folder, "folder_changed", folder_changed, mfi);
-	camel_object_hook_event(folder, "renamed", folder_renamed, mfi);
-	camel_object_hook_event(folder, "finalize", folder_finalised, mfi);
-
 	update_1folder(mfi, 0, NULL);
 
 	UNLOCK(info_lock);
+
+	camel_object_hook_event(folder, "folder_changed", folder_changed, NULL);
+	camel_object_hook_event(folder, "renamed", folder_renamed, NULL);
+	camel_object_hook_event(folder, "finalize", folder_finalised, NULL);
 }
 
 static void
@@ -527,6 +556,14 @@ store_folder_created(CamelObject *o, void *event_data, void *data)
 		store_folder_subscribed(o, event_data, data);
 }
 
+static void
+store_folder_opened(CamelObject *o, void *event_data, void *data)
+{
+	CamelFolder *folder = event_data;
+
+	mail_note_folder(folder);
+}
+
 static void
 store_folder_unsubscribed(CamelObject *o, void *event_data, void *data)
 {
@@ -559,10 +596,32 @@ store_folder_deleted(CamelObject *o, void *event_data, void *data)
 		store_folder_unsubscribed(o, event_data, data);
 }
 
+static char *
+folder_to_url(CamelStore *store, const char *full_name)
+{
+	CamelURL *url;
+	char *out;
+
+	url = camel_url_copy(((CamelService *)store)->url);
+	if (((CamelService *)store)->provider->url_flags  & CAMEL_URL_FRAGMENT_IS_PATH) {
+		camel_url_set_fragment(url, full_name);
+	} else {
+		char *name = g_alloca(strlen(full_name)+2);
+
+		sprintf(name, "/%s", full_name);
+		camel_url_set_path(url, name);
+	}
+
+	out = camel_url_to_string(url, CAMEL_URL_HIDE_ALL);
+	camel_url_free(url);
+
+	return out;
+}
+
 static void
 rename_folders(struct _store_info *si, const char *oldbase, const char *newbase, CamelFolderInfo *fi)
 {
-	char *old;
+	char *old, *olduri, *oldfile, *newuri, *newfile;
 	struct _folder_info *mfi;
 	struct _folder_update *up;
 
@@ -576,14 +635,12 @@ rename_folders(struct _store_info *si, const char *oldbase, const char *newbase,
 	if (mfi) {
 		d(printf("Found old folder '%s' renaming to '%s'\n", mfi->full_name, fi->full_name));
 
-		up->oldpath = mfi->path;
+		up->oldfull = mfi->full_name;
 		up->olduri = mfi->uri;
 
 		/* Its a rename op */
 		g_hash_table_remove(si->folders, mfi->full_name);
 		g_hash_table_remove(si->folders, mfi->uri);
-		g_free(mfi->full_name);
-		mfi->path = g_strdup(fi->path);
 		mfi->full_name = g_strdup(fi->full_name);
 		mfi->uri = g_strdup(fi->uri);
 		mfi->flags = fi->flags;
@@ -594,7 +651,6 @@ rename_folders(struct _store_info *si, const char *oldbase, const char *newbase,
 		d(printf("Rename found a new folder? old '%s' new '%s'\n", old, fi->full_name));
 		/* Its a new op */
 		mfi = g_malloc0(sizeof(*mfi));
-		mfi->path = g_strdup(fi->path);
 		mfi->full_name = g_strdup(fi->full_name);
 		mfi->uri = g_strdup(fi->uri);
 		mfi->store_info = si;
@@ -604,10 +660,7 @@ rename_folders(struct _store_info *si, const char *oldbase, const char *newbase,
 		g_hash_table_insert(si->folders_uri, mfi->uri, mfi);
 	}
 
-	g_free(old);
-
-	up->path = g_strdup(mfi->path);
-	up->name = g_strdup(fi->name);
+	up->full_name = g_strdup(mfi->full_name);
 	up->uri = g_strdup(mfi->uri);
 	up->unread = fi->unread==-1?0:fi->unread;
 	up->store = si->store;
@@ -624,6 +677,26 @@ rename_folders(struct _store_info *si, const char *oldbase, const char *newbase,
 	if (fi->child)
 		rename_folders(si, oldbase, newbase, fi->child, folders);
 #endif
+
+	/* rename the meta-data we maintain ourselves */
+	olduri = folder_to_url(si->store, old);
+	e_filename_make_safe(olduri);
+	newuri = folder_to_url(si->store, fi->full_name);
+	e_filename_make_safe(newuri);
+	oldfile = g_strdup_printf("%s/mail/config/custom_view-%s.xml", mail_component_peek_base_directory(NULL), olduri);
+	newfile = g_strdup_printf("%s/mail/config/custom_view-%s.xml", mail_component_peek_base_directory(NULL), newuri);
+	g_rename(oldfile, newfile);
+	g_free(oldfile);
+	g_free(newfile);
+	oldfile = g_strdup_printf("%s/mail/config/current_view-%s.xml", mail_component_peek_base_directory(NULL), olduri);
+	newfile = g_strdup_printf("%s/mail/config/current_view-%s.xml", mail_component_peek_base_directory(NULL), newuri);
+	g_rename(oldfile, newfile);
+	g_free(oldfile);
+	g_free(newfile);
+	g_free(olduri);
+	g_free(newuri);
+
+	g_free(old);
 }
 
 static void
@@ -645,7 +718,7 @@ folder_cmp(const void *ap, const void *bp)
 	const CamelFolderInfo *a = ((CamelFolderInfo **)ap)[0];
 	const CamelFolderInfo *b = ((CamelFolderInfo **)bp)[0];
 
-	return strcmp(a->path, b->path);
+	return strcmp(a->full_name, b->full_name);
 }
 
 static void
@@ -720,6 +793,7 @@ mail_note_store_remove(CamelStore *store)
 	if (si) {
 		g_hash_table_remove(stores, store);
 
+		camel_object_unhook_event(store, "folder_opened", store_folder_opened, NULL);
 		camel_object_unhook_event(store, "folder_created", store_folder_created, NULL);
 		camel_object_unhook_event(store, "folder_deleted", store_folder_deleted, NULL);
 		camel_object_unhook_event(store, "folder_renamed", store_folder_renamed, NULL);
@@ -741,6 +815,7 @@ mail_note_store_remove(CamelStore *store)
 		g_hash_table_destroy(si->folders_uri);
 		g_free(si);
 	}
+
 	UNLOCK(info_lock);
 }
 
@@ -750,7 +825,7 @@ update_folders(CamelStore *store, CamelFolderInfo *fi, void *data)
 	struct _update_data *ud = data;
 	struct _store_info *si;
 
-	d(printf("Got folderinfo for store\n"));
+	d(printf("Got folderinfo for store %s\n", store->parent_object.provider->protocol));
 
 	LOCK(info_lock);
 	si = g_hash_table_lookup(stores, store);
@@ -868,9 +943,10 @@ mail_note_store(CamelStore *store, CamelOperation *op,
 	struct _update_data *ud;
 	const char *buf;
 	guint timeout;
+	int hook = 0;
 	
 	g_assert(CAMEL_IS_STORE(store));
-	g_assert(pthread_self() == mail_gui_thread);
+	g_assert(pthread_equal(pthread_self(), mail_gui_thread));
 
 	LOCK(info_lock);
 
@@ -895,40 +971,49 @@ mail_note_store(CamelStore *store, CamelOperation *op,
 		camel_object_ref((CamelObject *)store);
 		g_hash_table_insert(stores, store, si);
 		e_dlist_init(&si->folderinfo_updates);
+		hook = TRUE;
+	}
+	
+	ud = g_malloc(sizeof(*ud));
+	ud->done = done;
+	ud->data = data;
+	ud->cancel = 0;
+	
+	/* We might get a race when setting up a store, such that it is still left in offline mode,
+	   after we've gone online.  This catches and fixes it up when the shell opens us */
+	if (CAMEL_IS_DISCO_STORE (store)) {
+		if (camel_session_is_online (session)
+		    && camel_disco_store_status (CAMEL_DISCO_STORE (store)) == CAMEL_DISCO_STORE_OFFLINE) {
+			/* Note: we use the 'id' here, even though its not the right id, its still ok */
+			ud->id = mail_store_set_offline (store, FALSE, store_online_cb, ud);
+		} else {
+			goto normal_setup;
+		}
+	} else if (CAMEL_IS_OFFLINE_STORE (store)) {
+		if (camel_session_is_online (session) && CAMEL_OFFLINE_STORE (store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+			/* Note: we use the 'id' here, even though its not the right id, its still ok */
+			ud->id = mail_store_set_offline (store, FALSE, store_online_cb, ud);
+		} else {
+			goto normal_setup;
+		}
+	} else {
+	normal_setup:
+		ud->id = mail_get_folderinfo (store, op, update_folders, ud);
+	}
+	
+	e_dlist_addtail (&si->folderinfo_updates, (EDListNode *) ud);
+	
+	UNLOCK(info_lock);
 
+	/* there is potential for race here, but it is safe as we check for the store anyway */
+	if (hook) {
+		camel_object_hook_event(store, "folder_opened", store_folder_opened, NULL);
 		camel_object_hook_event(store, "folder_created", store_folder_created, NULL);
 		camel_object_hook_event(store, "folder_deleted", store_folder_deleted, NULL);
 		camel_object_hook_event(store, "folder_renamed", store_folder_renamed, NULL);
 		camel_object_hook_event(store, "folder_subscribed", store_folder_subscribed, NULL);
 		camel_object_hook_event(store, "folder_unsubscribed", store_folder_unsubscribed, NULL);
 	}
-
-	/* We might get a race when setting up a store, such that it is still left in offline mode,
-	   after we've gone online.  This catches and fixes it up when the shell opens us */
-	if (CAMEL_IS_DISCO_STORE(store)
-	    && camel_session_is_online(session)
-	    && camel_disco_store_status (CAMEL_DISCO_STORE (store)) == CAMEL_DISCO_STORE_OFFLINE) {
-		ud = g_malloc(sizeof(*ud));
-		ud->done = done;
-		ud->data = data;
-		ud->cancel = 0;
-		/* Note: we use the 'id' here, even though its not the right id, its still ok */
-		ud->id = mail_store_set_offline (store, FALSE, store_online_cb, ud);
-
-		e_dlist_addtail (&si->folderinfo_updates, (EDListNode *) ud);
-	} else if (!CAMEL_IS_DISCO_STORE(store)
-		   || camel_disco_store_status (CAMEL_DISCO_STORE (store)) == CAMEL_DISCO_STORE_ONLINE
-		   || camel_disco_store_can_work_offline (CAMEL_DISCO_STORE (store))) {
-		ud = g_malloc (sizeof (*ud));
-		ud->done = done;
-		ud->data = data;
-		ud->cancel = 0;
-		ud->id = mail_get_folderinfo (store, op, update_folders, ud);
-
-		e_dlist_addtail (&si->folderinfo_updates, (EDListNode *) ud);
-	}
-
-	UNLOCK(info_lock);
 }
 
 struct _find_info {
