@@ -34,7 +34,7 @@
 
 #include "mail/em-account-editor.h"
 #include "mail/em-config.h"
-#include "e-util/e-account.h"
+#include "libedataserver/e-account.h"
 #include "e-util/e-error.h"
 
 #include "exchange-operations.h"
@@ -146,9 +146,11 @@ e_exchange_contacts_pcontacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 	}
 
         uri_text = e_source_get_uri (source);
-	if (uri_text && strncmp (uri_text, "exchange", 8)) {
-		g_free (uri_text);		
-		return NULL;
+	if (uri_text && g_ascii_strncasecmp (uri_text, "exchange", 8)) {
+		if (g_ascii_strncasecmp (uri_text, "gal", 3)) {
+			g_free (uri_text);
+			return NULL;
+		}
 	}
 
 	g_free (uri_text);
@@ -308,6 +310,7 @@ e_exchange_contacts_commit (EPlugin *epl, EConfigTarget *target)
 	EABConfigTargetSource *t = (EABConfigTargetSource *) target;
 	ESource *source = t->source;
 	gchar *uri_text, *gname, *gruri, *ruri, *path, *path_prefix, *oldpath=NULL;
+	gchar *username, *authtype;
 	int prefix_len;
 	ExchangeAccount *account;
 	ExchangeAccountFolderResult result;
@@ -318,42 +321,60 @@ e_exchange_contacts_commit (EPlugin *epl, EConfigTarget *target)
 		g_free (uri_text);
 		return ;
 	}	
-	g_free (uri_text);
 
 	exchange_config_listener_get_offline_status (exchange_global_config_listener, 
 								    &offline_status);
-	if (offline_status == OFFLINE_MODE)
+	if (offline_status == OFFLINE_MODE) {
+		g_free (uri_text);
 		return;
+	}
 
 	account = exchange_operations_get_exchange_account ();
+	username = exchange_account_get_username (account);
+	authtype = exchange_account_get_authtype (account);
+
 	path_prefix = g_strconcat (account->account_filename, "/;", NULL);
 	prefix_len = strlen (path_prefix);
 	g_free (path_prefix);
 
 	gname = (gchar*) e_source_peek_name (source);
 	gruri = (gchar*) e_source_peek_relative_uri (source);
+
 	if (contacts_src_exists) {
-		gchar *tmpruri, *tmpdelimit;
-		tmpruri = g_strdup (gruri);
-		tmpdelimit = g_strrstr (tmpruri, "/");
-		tmpdelimit[0] = '\0';
-		ruri = g_strconcat (tmpruri, "/", gname, NULL);
+		gchar *tmpruri, *uri_string;
+		EUri *euri;
+		int uri_len;
+
+		euri = e_uri_new (uri_text);
+		uri_string = e_uri_to_string (euri, FALSE);
+		e_uri_free (euri);
+	
+		uri_len = strlen (uri_string) + 1;	
+		tmpruri = g_strdup (uri_string + strlen ("exchange://"));
+		ruri = g_strconcat (tmpruri, uri_text + uri_len, NULL);
+		path = g_build_filename ("/", uri_text + uri_len, NULL);
+		oldpath = g_build_filename ("/", contacts_old_src_uri + prefix_len, NULL);
+		g_free (uri_string);
 		g_free (tmpruri);
 	}
 	else {
 		ruri = g_strconcat (gruri, "/", gname, NULL);
+		path = g_build_filename ("/", ruri+prefix_len, NULL);
 	}
-	e_source_set_relative_uri (source, ruri);
 
-	path = g_strdup_printf ("/%s", ruri+prefix_len);
-	
+	e_source_set_relative_uri (source, ruri);
+	e_source_set_property (source, "username", username);
+	e_source_set_property (source, "auth-domain", "Exchange");
+	if (authtype)
+		e_source_set_property (source, "auth-type", authtype);
+	e_source_set_property (source, "auth", "plain/password");
+
 	if (!contacts_src_exists) {
 		/* Create the new folder */
 		result = exchange_account_create_folder (account, path, "contacts");
 	}
-	else if (strcmp (gruri, contacts_old_src_uri)) {
+	else if (gruri && strcmp (gruri, contacts_old_src_uri) && strcmp (path, oldpath)) {
 		/* Rename the folder */
-		oldpath = g_strdup_printf ("/%s", contacts_old_src_uri+prefix_len);
 		result = exchange_account_xfer_folder (account, oldpath, path, TRUE);
 		exchange_operations_update_child_esources (source, 
 							   contacts_old_src_uri, 
@@ -361,7 +382,7 @@ e_exchange_contacts_commit (EPlugin *epl, EConfigTarget *target)
 	}
 	else {
 		/* Nothing happened specific to exchange; just return */
-		return;
+		goto done;
 	}
 
 	switch (result) {
@@ -389,6 +410,7 @@ e_exchange_contacts_commit (EPlugin *epl, EConfigTarget *target)
 	default:
 		break;
 	}
+done:
 	g_free (ruri);
 	g_free (path);
 	g_free (oldpath);
