@@ -20,26 +20,16 @@
  * Author: Ettore Perazzoli <ettore@ximian.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
-#include "e-shell-window-commands.h"
-
-#include "e-shell-importer.h"
-#include "e-shell-window.h"
-
-#include "evolution-shell-component-utils.h"
-
-#include "e-util/e-icon-factory.h"
-#include "e-util/e-dialog-utils.h"
-#include "e-util/e-error.h"
+#include <string.h>
 
 #include <glib/gprintf.h>
 
 #include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-url.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-url.h>
+
 #include <libgnomeui/gnome-about.h>
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -50,7 +40,18 @@
 
 #include <libedataserverui/e-passwords.h>
 
-#include <string.h>
+#include <gconf/gconf-client.h>
+
+#include "e-util/e-icon-factory.h"
+#include "e-util/e-dialog-utils.h"
+#include "e-util/e-error.h"
+#include "e-util/e-util-private.h"
+
+#include "e-shell-window-commands.h"
+#include "e-shell-window.h"
+#include "evolution-shell-component-utils.h"
+
+#include "e-shell-importer.h"
 
 /* Utility functions.  */
 
@@ -158,7 +159,6 @@ static const char *authors[] = {
 	"Ali Abdin",
 	"Almer S. Tigelaar",
 	"Amish",	
-	"A Nagappan",
 	"Anders Carlsson",
 	"Andrea Campi",
 	"Andreas Hyden",
@@ -179,7 +179,7 @@ static const char *authors[] = {
 	"Benjamin Kahn",
 	"Bertrand Guiheneuf",
 	"Bill Zhu",
-	"Bjorn Torkelsson"
+	"Bjorn Torkelsson",
 	"Bob Doan",
 	"Bolian Yin",
 	"Brian Mury",
@@ -252,7 +252,6 @@ static const char *authors[] = {
 	"Federico Mena Quintero",
 	"Fernando Herrera",
 	"Francisco Javier F. Serrador",
-	"Francisco Javier F. Serrador",
 	"Frank Belew",
 	"Frederic Crozat",
 	"Gareth Owen",
@@ -283,7 +282,7 @@ static const char *authors[] = {
 	"Ian Campbell",
 	"Ismael Olea",
 	"Israel Escalante",
-	"Iv\xEF\xBF\xBD" "n Frade",
+	"Iv\xC3\xA1" "n Frade",
 	"Jack Jia",
 	"Jacob Berkman",
 	"Jaka Mocnik",
@@ -305,7 +304,6 @@ static const char *authors[] = {
 	"Jeremy Wise",
 	"Jerome Lacoste",
 	"Jesse Pavel",
-	"Jesus Bravo Alvarez",
 	"Jes\xC3\xBA" "s Bravo \xC3\x81" "lvarez",
 	"J.H.M. Dassen (Ray)",
 	"Ji Lee",
@@ -314,7 +312,6 @@ static const char *authors[] = {
 	"Joe Shaw",
 	"John Gotts",
 	"Jonas Borgstr",
-	"Jonas Borgstr",
 	"Jonathan Blandford",
 	"Jonathan Dieter",
 	"Jon K Hellan",
@@ -388,6 +385,7 @@ static const char *authors[] = {
 	"Mohammad DAMT",
 	"Morten Welinder",
 	"Naba Kumar",
+	"Nagappan Alagappan",
 	"Nat Friedman",
 	"Nicel KM",
 	"Nicholas J Kreucher",
@@ -576,6 +574,8 @@ command_about_box (BonoboUIComponent *uih,
 	gtk_widget_show (about_box_window);
 }
 
+#if 0
+/* Unused */
 static void
 command_help_faq (BonoboUIComponent *uih,
 		  EShellWindow *window,
@@ -585,6 +585,7 @@ command_help_faq (BonoboUIComponent *uih,
 	/* FIXME use the error */
 	gnome_url_show ("http://gnome.org/projects/evolution/faq.shtml", NULL);	
 }
+#endif
 
 static void
 command_quick_reference (BonoboUIComponent *uih,
@@ -636,7 +637,7 @@ command_work_offline (BonoboUIComponent *uih,
 		      EShellWindow *window,
 		      const char *path)
 {
-	e_shell_go_offline (e_shell_window_peek_shell (window), window);
+	e_shell_go_offline (e_shell_window_peek_shell (window), window, GNOME_Evolution_USER_OFFLINE);
 }
 
 static void
@@ -644,7 +645,7 @@ command_work_online (BonoboUIComponent *uih,
 		     EShellWindow *window,
 		     const char *path)
 {
-	e_shell_go_online (e_shell_window_peek_shell (window), window);
+	e_shell_go_online (e_shell_window_peek_shell (window), window, GNOME_Evolution_USER_ONLINE);
 }
 
 static void
@@ -904,6 +905,39 @@ view_toolbar_item_toggled_handler (BonoboUIComponent           *ui_component,
 				      "hidden", is_visible ? "0" : "1", NULL);
 }
 
+static void
+view_statusbar_item_toggled_handler (BonoboUIComponent           *ui_component,
+				     const char                  *path,
+				     Bonobo_UIComponent_EventType type,
+				     const char                  *state,
+				     EShellWindow                *shell_window)
+{
+	GtkWidget *status_bar = e_shell_window_peek_statusbar (shell_window);
+	gboolean is_visible;
+	is_visible = state[0] == '1';	
+	if(is_visible)
+		gtk_widget_show (status_bar);
+	else
+		gtk_widget_hide (status_bar);
+	gconf_client_set_bool (gconf_client_get_default (),"/apps/evolution/shell/view_defaults/statusbar_visible", is_visible, NULL);
+}
+
+static void
+view_sidebar_item_toggled_handler (BonoboUIComponent           *ui_component,
+				     const char                  *path,
+				     Bonobo_UIComponent_EventType type,
+				     const char                  *state,
+				     EShellWindow                *shell_window)
+{
+	GtkWidget *side_bar = GTK_WIDGET(e_shell_window_peek_sidebar (shell_window));
+	gboolean is_visible;
+	is_visible = state[0] == '1';	
+	if(is_visible)
+		gtk_widget_show (side_bar);
+	else
+		gtk_widget_hide (side_bar);
+	gconf_client_set_bool (gconf_client_get_default (),"/apps/evolution/shell/view_defaults/sidebar_visible", is_visible, NULL);
+}
 
 /* Public API.  */
 
@@ -942,6 +976,12 @@ e_shell_window_commands_setup (EShellWindow *shell_window)
 	bonobo_ui_component_add_listener (uic, "ViewToolbar",
 					  (BonoboUIListenerFn)view_toolbar_item_toggled_handler,
 					  (gpointer)shell_window);
+	bonobo_ui_component_add_listener (uic, "ViewStatusBar",
+					  (BonoboUIListenerFn)view_statusbar_item_toggled_handler,
+					  (gpointer)shell_window);
+	bonobo_ui_component_add_listener (uic, "ViewSideBar",
+					  (BonoboUIListenerFn)view_sidebar_item_toggled_handler,
+					  (gpointer)shell_window);
 
 	e_pixmaps_update (uic, pixmaps);
 
