@@ -258,7 +258,7 @@ show_development_warning(void)
 
 	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 
-	dont_bother_me_again_checkbox = gtk_check_button_new_with_label (_("Don't tell me again"));
+	dont_bother_me_again_checkbox = gtk_check_button_new_with_label (_("Do not tell me again"));
 
 	alignment = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
 
@@ -328,7 +328,6 @@ idle_cb (void *data)
 	CORBA_Environment ev;
 	EShellConstructResult result;
 	EShellStartupLineMode startup_line_mode;
-	gboolean have_evolution_uri;
 
 #ifdef KILL_PROCESS_CMD
 	kill_old_dataserver ();
@@ -374,8 +373,6 @@ idle_cb (void *data)
 
 	}
 
-	have_evolution_uri = FALSE;
-
 	if (shell != NULL) {
 		e_shell_create_window (shell, default_component_id, NULL);
 		open_uris (corba_shell, uri_list);
