@@ -28,21 +28,17 @@
 
 #include "e-clipped-label.h"
 
-#include <gal/e-table/e-table-scrolled.h>
-#include <gal/e-table/e-table-memory-store.h>
-#include <gal/e-table/e-cell-pixbuf.h>
-#include <gal/e-table/e-cell-vbox.h>
-#include <gal/e-table/e-cell-text.h>
+#include <table/e-table-scrolled.h>
+#include <table/e-table-memory-store.h>
+#include <table/e-cell-pixbuf.h>
+#include <table/e-cell-vbox.h>
+#include <table/e-cell-text.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 
-#define PARENT_TYPE gtk_dialog_get_type ()
-static GtkDialogClass *parent_class = NULL;
-
 #define SWITCH_PAGE_INTERVAL 250
 
-
 struct _EMultiConfigDialogPrivate {
 	GSList *pages;
 	
@@ -55,6 +51,8 @@ struct _EMultiConfigDialogPrivate {
 	int set_page_timeout_page;
 };
 
+G_DEFINE_TYPE (EMultiConfigDialog, e_multi_config_dialog, GTK_TYPE_DIALOG)
+
 
 /* ETable stuff.  */
 
@@ -70,7 +68,7 @@ static char *list_e_table_spec =
 	"                cell=\"vbox\""
  	"                minimum_width=\"32\""
 	"                resizable=\"true\""
-	"	         _title=\"blah\""
+	"	         _title=\"Category\""
 	"                compare=\"string\"/>"
 	"  <ETableState>"
 	"    <column source=\"0\"/>"
@@ -124,6 +122,7 @@ set_page_timeout_callback (void *data)
 	gtk_notebook_set_page (GTK_NOTEBOOK (priv->notebook), priv->set_page_timeout_page);
 
 	priv->set_page_timeout_id = 0;
+	gtk_widget_grab_focus(priv->list_e_table);
 	return FALSE;
 }
 
@@ -177,7 +176,7 @@ impl_finalize (GObject *object)
 
 	g_free (priv);
 
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (e_multi_config_dialog_parent_class)->finalize) (object);
 }
 
 
@@ -216,7 +215,7 @@ impl_response (GtkDialog *dialog, int response_id)
 /* GObject ctors.  */
 
 static void
-class_init (EMultiConfigDialogClass *class)
+e_multi_config_dialog_class_init (EMultiConfigDialogClass *class)
 {
 	GObjectClass *object_class;
 	GtkDialogClass *dialog_class;
@@ -226,8 +225,6 @@ class_init (EMultiConfigDialogClass *class)
 
 	dialog_class = GTK_DIALOG_CLASS (class);
 	dialog_class->response = impl_response;
-
-	parent_class = g_type_class_ref (PARENT_TYPE);
 }
 
 #define RGB_COLOR(color) (((color).red & 0xff00) << 8 | \
@@ -287,7 +284,7 @@ static ETableMemoryStoreColumnInfo columns[] = {
 };
 
 static void
-init (EMultiConfigDialog *multi_config_dialog)
+e_multi_config_dialog_init (EMultiConfigDialog *multi_config_dialog)
 {
 	EMultiConfigDialogPrivate *priv;
 	ETableModel *list_e_table_model;
@@ -449,5 +446,3 @@ e_multi_config_dialog_show_page (EMultiConfigDialog *dialog, int page)
 	gtk_notebook_set_page (GTK_NOTEBOOK (priv->notebook), page);
 }
 
-
-E_MAKE_TYPE (e_multi_config_dialog, "EMultiConfigDialog", EMultiConfigDialog, class_init, init, PARENT_TYPE)
