@@ -181,7 +181,7 @@ find_model_column_by_name (ETableSpecification *spec, const char *s)
 
 		if ((*column)->disabled)
 			continue;
-		if (g_strcasecmp ((*column)->title, s) == 0)
+		if (g_ascii_strcasecmp ((*column)->title, s) == 0)
 			return (*column)->model_col;
 	}
 	return -1;
@@ -363,8 +363,8 @@ setup_fields (ETableConfig *config)
 {
 	int i;
 
-	e_table_model_freeze (config->available_model);
-	e_table_model_freeze (config->shown_model);
+	e_table_model_freeze ((ETableModel *)config->available_model);
+	e_table_model_freeze ((ETableModel *)config->shown_model);
 	e_table_without_show_all (config->available_model);
 	e_table_subset_variable_clear (config->shown_model);
 
@@ -379,8 +379,8 @@ setup_fields (ETableConfig *config)
 			e_table_without_hide (config->available_model, GINT_TO_POINTER(idx));
 		}
 	}
-	e_table_model_thaw (config->available_model);
-	e_table_model_thaw (config->shown_model);
+	e_table_model_thaw ((ETableModel *)config->available_model);
+	e_table_model_thaw ((ETableModel *)config->shown_model);
 }
 
 static void
@@ -553,10 +553,16 @@ GtkWidget *
 e_table_proxy_etable_shown_new (void)
 {
 	ETableModel *model = NULL;
+	GtkWidget *widget;
+	ETableScrolled *ets;
 
 	model = e_table_subset_variable_new (global_store);
 
-	return e_table_scrolled_new (model, NULL, spec, NULL);
+	widget = e_table_scrolled_new (model, NULL, spec, NULL);
+	ets = E_TABLE_SCROLLED (widget);
+	atk_object_set_name (gtk_widget_get_accessible ((GtkWidget *)ets->table), _("Show Fields"));
+
+	return widget;
 }
 
 GtkWidget *e_table_proxy_etable_available_new (void);
@@ -565,13 +571,19 @@ GtkWidget *
 e_table_proxy_etable_available_new (void)
 {
 	ETableModel *model;
+	GtkWidget *widget;
+	ETableScrolled *ets;
 
 	model = e_table_without_new (global_store,
 				     NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
 	e_table_without_show_all (E_TABLE_WITHOUT (model));
 
-	return e_table_scrolled_new (model, NULL, spec, NULL);
+	widget = e_table_scrolled_new (model, NULL, spec, NULL);
+	ets = E_TABLE_SCROLLED (widget);
+	atk_object_set_name (gtk_widget_get_accessible ((GtkWidget *)ets->table), _("Available Fields"));
+
+	return widget;
 }
 
 static void
